!------------------------------------------------------------------------
! This is the main program for the package SPheno, version 3.0beta
! SPheno stands for S(upersymmetric) Pheno(menology) 
! A documentation is provided in the file Spheno.ps
! Please send any comments and/or bugs to porod@physik.unizh.ch
! In case of a bug:
!   - please check that the integer in the first line of Control.in
!     is 0 or larger. If it is negative, please set it to 0. This
!     enables the writing of several warnings. Afterwards rerun the 
!     program.
!   - please send the file Error.out  and all input files (*.in) to
!     the email above, if possible with a copy of the screen output 
! written by Werner Porod
!-----------------------------------------------------------------
Program SPheno

!---------------------------------
! loading necessary modules
!---------------------------------
Use BranchingRatios
Use Chargino3Decays
Use Control
Use EplusEminusProduction
Use InputOutput
Use LoopFunctions
Use LoopMasses
Use LowEnergy
Use Mathematics
Use Model_Data
Use RPtools
Use RGEs
Use StandardModel
Use SugraRuns

! Use f90_unix_proc
 Implicit None
 !-------------------------------
 ! widths and branching ratios
 !-------------------------------
  Real(dp) ::  epsI, deltaM
 !--------------------------------
 ! cross section calculation
 !--------------------------------
 Integer, Parameter :: p_max=100
 Complex(dp) :: Ylp(3,3)
 Real(dp) :: Ecms(p_max), Pm(p_max), Pp(p_max), SigSup(p_max,6,6)        &
         & , SigSdown(p_max,6,6), SigSle(p_max,6,6), SigSn(p_max,3,3)    &
         & , SigC(p_max,5,5), SigChi0(p_max,7,7), SigS0(p_max,5)         &
         & , SigSP(p_max,5,4), SigHp(p_max,7,7)
 Logical :: ISR(p_max)=.False., Beam(p_max)=.False.
 !----------------------------------
 ! low energy constraints
 !----------------------------------
 Real(dp) :: BRbtosgamma, Bs_mumu, BrBToSLL, BR_Bu_TauNu, a_e, a_mu, a_tau &
   & , d_e, d_mu, d_tau, BrMutoEGamma, BrTautoEGamma, BrTautoMuGamma       &
   & , BrMu3e, BrTau3e, BrTau3Mu, BR_Z_e_mu, BR_Z_e_tau, BR_Z_mu_tau, BtoSNuNu
 Complex(dp) :: DeltaMBd, DeltaMBs
 !------------------------------------
 ! auxiliary parameters
 !------------------------------------
 Logical :: CalcTBD
 Integer :: kont, i1, i_min(3),i2,i3,i4
 Real(dp) :: m_Gut, ratioWoM,xx,yy,zz,aa

 !----------------------------------
 ! Dark Matter
 !----------------------------------

 Real(dp) :: omega

 !--------------------------------------------------------------------------
 ! set all parameters to zero
 !--------------------------------------------------------------------------
 Call Set_All_Parameters_0()

 !--------------------------------------------------------------------------
 ! This routine call routines to
 !   - initializ the error system
 !   - to calculate the constants which are required for the 
 !     calculation of the loop functions
 !   - to get the standard model parameters
 !   - to get SUSY parameters
 ! The following steps are performed to get the parameters and flags
 !   (i) The file LesHouches.in exists containing all necessary information.
 !       In this case the remaining steps are skipped
 !   (ii) Check if Control.in exists to set the error level and
 !        to check if widths and cross sections shall be calculated
 !   (iii) Check if StandardModel.in exists to change the default SM values
 !   (iv) Read the information concerning the SUSY model from the file
 !        HighScale.in
 ! Note that either the file LesHouches.in or the file HighScale.in
 ! must exist.
 !--------------------------------------------------------------------------
 kont = 0
 Call ReadingData(kont)

! Do i1=-30,30
!  xx = 10._dp**i1
!  Write(10,*) i1,F_1_0(xx),F_2_0(xx),F_8_0(xx)
!  Do i2=-30,30
!   yy = 10._dp**i2
!   Write(10,*) i1,i2,F_3_0(xx,yy),F_4_0(xx,yy),F_7_0(xx,yy),F_11_0(xx,yy)
!    Do i3=-30,30
!     zz = 10._dp**i3
! Write(*,*) xx,yy,zz
!     Write(10,*) i1,i2,i3,F_5_0(xx,yy,zz),F_6_0(xx,yy,zz)
!     Do i4=-30,30
!      aa = 10._dp**i4
! Write(*,*) i1,i2,i3,i4
!      Write(10,*) i1,i2,i4,F_9_0(xx,yy,zz,aa),F_10_0(xx,yy,zz,aa)
!     end do
!    end do
!  end do
! end do
!stop 99
 !---------------------------------------------------------------------------
 ! calculation of the spectrum, the following parameters can be changed
 ! with the help of the SLHA input file LesHouches.in or the file Control.in
 ! 
 !---------------------------------------------------------------------------
 If (kont.Eq.0) Call CalculateSpectrum(n_run, delta_mass, WriteOut, kont, tanb &
    & , vevSM, mC, U, V, mN, N, mS0, mS02, RS0, mP0, mP02, RP0, mSpm, mSpm2    &
    & , RSpm, mSdown, mSdown2, RSdown, mSup, mSup2, RSup, mSlepton, mSlepton2  &
    & , RSlepton, mSneut, mSneut2, RSneut, mGlu, PhaseGlu, gauge               &
    & , uL_L, uL_R, uD_L, uD_R, uU_L, uU_R, Y_l, Y_d, Y_u                      &
    & , Mi, A_l, A_d, A_u, M2_E, M2_L, M2_D, M2_Q, M2_U, M2_H, mu, B           &
    & , m_GUT)

 !-------------------------------------------------------------------
 ! add R-parity violation to the model respecting neutrino data
 ! neglects currently flavour violation
 !-------------------------------------------------------------------
 If (Add_Rparity) Then

  Call Calculate_RP_Parameters(delta_mass, eps, vevL, Beps, Lam_ex, RSpm8 &
                            & , RS05, RP05, U5, V5, mN7, N7, kont)
  mC5(1:3) = mf_l
  mC5(4:5) = mC
  mN7(4:7) = mN
  mP05(1) = mZ
  Do i1=2,5
   If ( (RP05(i1,1)**2 +RP05(i1,2)**2).Gt.0.5_dp) mP05(i1) = mP0(2)
   If ( RP05(i1,3)**2.Gt.0.5_dp) mP05(i1) = mSneut(1)
   If ( RP05(i1,4)**2.Gt.0.5_dp) mP05(i1) = mSneut(2)
   If ( RP05(i1,5)**2.Gt.0.5_dp) mP05(i1) = mSneut(3)
  End Do
  mP052 = mP05**2
  i_min = 0
  Do i1=1,5
   If ( (RS05(i1,1)**2 +RS05(i1,2)**2).Gt.0.5_dp) Then
    mS05(i1) = mS0(1 + i_min(1) )
    i_min(1) = i_min(1) + 1
   End If
   If ( RS05(i1,3)**2.Gt.0.5_dp) mS05(i1) = mSneut(1)
   If ( RS05(i1,4)**2.Gt.0.5_dp) mS05(i1) = mSneut(2)
   If ( RS05(i1,5)**2.Gt.0.5_dp) mS05(i1) = mSneut(3)
  End Do
  mS052 = mS05**2
  mSpm8(1) = mW
  i_min = 0
  Do i1=2,8
   If ( (Abs(RSpm8(i1,1))**2 +Abs(RSpm8(i1,2))**2).Gt.0.5_dp) &
      &  mSpm8(i1) = mSpm(2)
   If ( (Abs(RSpm8(i1,3))**2 +Abs(RSpm8(i1,6))**2).Gt.0.5_dp) Then
    mSpm8(i1) = mSlepton(1 + i_min(1) )
    i_min(1) = i_min(1) + 1
   End If
   If ( (Abs(RSpm8(i1,4))**2 +Abs(RSpm8(i1,7))**2).Gt.0.5_dp) Then
    mSpm8(i1) = mSlepton(3 + i_min(2) )
    i_min(2) = i_min(2) + 1
   End If
   If ( (Abs(RSpm8(i1,5))**2 +Abs(RSpm8(i1,8))**2).Gt.0.5_dp) Then
    mSpm8(i1) = mSlepton(5 + i_min(3) )
    i_min(3) = i_min(3) + 1
   End If
  End Do
  mSpm82 = mSpm8**2

  HighScaleModel = "RPexplicit"
 End If
 !-----------------------------------------------------------------------
 ! as there is a large hierachy between m_nu and m_chi, there might have
 ! been numerical problems. However, in all known cases this has been
 ! the mass of the lightest neutrino, which is of no importance for the
 ! following. Therefore, the error flag is reset to zero. 
 ! This is of course dangerous and has to be improved.
 !-----------------------------------------------------------------------
 If (kont.Eq.-8) kont = 0

 !-------------------------------------------------------------------
 ! Calculation of the branching ratios and widths provided L_BR is
 ! set .TRUE. (default) and that the routine Sugra has finished
 ! correctly (kont.eq.0) 
 !-------------------------------------------------------------------

 If ((L_BR).And.(kont.Eq.0)) Then
  ! relative precision for the calculation of phase space integrals
  epsI = 1.e-5_dp
  deltaM = 1.e-3_dp ! if mass/phase space is smaller, than mass is treated as 0
  CalcTBD = .False. ! if .True. than calculation of 3-body decays is enforced
  ratioWoM = 0._dp ! 1.e-4_dp

  If (HighScaleModel.Eq."NMSSM") Then

   Call CalculateBR(gauge, mGlu, PhaseGlu, mC, U, V, mN5, N5, mSneut, RSneut &
     & , mSlepton, RSlepton, mSup, RSup, mSdown, RSdown, uL_L, uL_R          &
     & , uD_L, uD_R, uU_L, uU_R, mS03, RS03, mP03, RP03, mSpm, RSpm, epsI    &
     & , deltaM, CalcTBD, kont, ratioWoM, Y_d, A_d, Y_l, A_l, Y_u, A_u       &
     & , mu, h0, A_h0, lam, A_lam, vevSM, vP, F_Gmsb, m32                    &
     & , gP_Sl, gT_Sl, BR_Sl, gP_Sn, gT_Sn, BR_Sn, gP_Sd, gT_Sd, BR_Sd       &
     & , gP_Su, gT_Su, BR_Su, gP_C, gT_C, BR_C, gP_N5, gT_N5, BR_N5          &
     & , gP_Glu, gT_Glu, BR_Glu, gP_P03, gT_P03, BR_P03                      &
     & , gP_S03, gT_S03, BR_S03, gP_Spm, gT_Spm, BR_Spm)

  Else If (HighScaleModel.Eq."RPexplicit") Then

   Call CalculateBR(gauge, mGlu, PhaseGlu, mC5, U5, V5, mN7, N7, mSup, RSup  &
     & , mSdown, RSdown, uD_L, uD_R, uU_L, uU_R, mS05, RS05, mP05, RP05      &
     & , mSpm8, RSpm8, epsI, deltaM, CalcTBD, kont, ratioWoM, Y_d, A_d, Y_l  &
     & , A_l, Y_u, A_u, mu, eps, vevSM, vevL, F_Gmsb, m32                    &
     & , gP_Sd, gT_Sd, BR_Sd, gP_Su, gT_Su, BR_Su                            &
     & , gT_C, gP_C2, BR_C2, gP_C3, BR_C3                                    &
     & , gT_N, gP_N4_2, BR_N4_2, gP_N4_3, BR_N4_3                            &
     & , gP_Glu, gT_Glu, BR_Glu, gP_P05, gT_P05, BR_P05                      &
     & , gP_S05, gT_S05, BR_S05, gP_Spm8, gT_Spm8, BR_Spm8)

  Else

   Call CalculateBR(gauge, mGlu, PhaseGlu, mC, U, V, mN, N, mSneut, RSneut  &
     & , mSlepton, RSlepton, mSup, RSup, mSdown, RSdown, uL_L, uL_R         &
     & , uD_L, uD_R, uU_L, uU_R, mS0, RS0, mP0, RP0, mSpm, RSpm, epsI       &
     & , deltaM, CalcTBD, kont, ratioWoM, Y_d, A_d, Y_l, A_l, Y_u, A_u, mu  &
     & , vevSM, F_Gmsb, m32, gP_Sl, gT_Sl, BR_Sl, gP_Sn, gT_Sn, BR_Sn       &
     & , gP_Sd, gT_Sd, BR_Sd, gP_Su, gT_Su, BR_Su, gP_C, gT_C, BR_C         &
     & , gT_N, gP_N4_2, BR_N4_2, gP_N4_3, BR_N4_3           &
     & , gP_Glu, gT_Glu, BR_Glu, gP_P0, gT_P0, BR_P0      &
     & , gP_S0, gT_S0, BR_S0, gP_Spm, gT_Spm, BR_Spm)

   End If
  End If


 !---------------------------------------------------------------------------
 ! Calculation of the cross sections in e+ e- annihilation provided L_Cs is
 ! set .TRUE. (default) and that the routine Sugra has finished
 ! correctly (kont.eq.0) 
 ! The following input quantities can be specified in the file 
 ! CrossSections.in: Ecms .... c.m.s enerergy in GeV
 !                   Pm ...... degree of longitudinal polarization of incoming
 !                             electron
 !                   Pp ...... degree of longitudinal polarization of incoming
 !                             positron
 !                   ISR ..... if .TRUE. then the effect of initial state
 !                             radiation will be included
 ! In the case that the file CrossSections.in does not exist, the following
 ! default values are used: Ecms = 500 GeV, Pm = Pp = 0, ISR = .TRUE.
 !----------------------------------------------------------------------------
 If ((L_CS).And.(kont.Eq.0)) Then
  If (HighScaleModel.Eq."NMSSM") Then
    Ylp = Y_l / gauge(2)
    Do i1=1,p_max
     If (Ecms(i1).Eq.0._dp) Exit
     Call CalculateCrossSections(Ecms(i1), Pm(i1), Pp(i1), ISR(i1), Beam(i1)   &
            & , "Tesla800", mSup, RSup, mf_u, mSdown, RSdown, mf_d, mglu       &
            & , SigSup(i1,:,:), SigSdown(i1,:,:), mSlepton, RSlepton, Ylp      &
            & , mSneut, RSneut, SigSle(i1,:,:), SigSn(i1,:,:), mC, U, V        &
            & , mN5, N5, SigC(i1,:,:), SigChi0(i1,1:5,1:5), mS03, RS03, vevSM  &
            & , mP03, RP03, mSpm, RSpm, SigS0(i1,1:3), SigSP(i1,1:3,1:2)       &
            & , SigHp(i1,1,1) )
    End Do
 
  Else If (HighScaleModel.Eq."RPexplicit") Then

   Do i1=1,p_max
    If (Ecms(i1).Eq.0._dp) Exit
    Call CalculateCrossSections(Ecms(i1), Pm(i1), Pp(i1), ISR(i1), Beam(i1) &
           & , "Tesla800", mSup, RSup, mf_u, mSdown, RSdown, mf_d, mglu     &
           & , SigSup(i1,:,:), SigSdown(i1,:,:), mC5, U5, V5, mN7, N7       &
           & , SigC(i1,:,:), SigChi0(i1,:,:), mS05, RS05, vevSM, vevL       &
           & , mP05, RP05, mSpm8, RSpm8, SigS0(i1,:), SigSP(i1,:,:)         &
           & , SigHp(i1,:,:) )
   End Do

  Else ! models with MSSM low energy parameters and spectrum
   Ylp = Y_l / gauge(2)
   Do i1=1,p_max
    If (Ecms(i1).Eq.0._dp) Exit
    Call CalculateCrossSections(Ecms(i1), Pm(i1), Pp(i1), ISR(i1), Beam(i1)  &
           & , "Tesla800", mSup, RSup, mf_u, mSdown, RSdown, mf_d, mglu      &
           & , SigSup(i1,:,:), SigSdown(i1,:,:), mSlepton, RSlepton, Ylp     &
           & , mSneut, RSneut, SigSle(i1,:,:), SigSn(i1,:,:), mC, U, V       &
           & , mN, N, SigC(i1,1:2,1:2), SigChi0(i1,1:4,1:4), mS0, RS0, vevSM &
           & , mP0, RP0, mSpm, RSpm, SigS0(i1,1:2), SigSP(i1,1:2,1)          &
           & , SigHp(i1,1,1) )
   End Do

  End If
 End If

 If ((kont.Eq.0).And.(HighScaleModel.Ne."NMSSM")) Then

  Call CalculateLowEnergyConstraints(gauge, Y_l, Y_d, Y_u      &
    & , mSpm2, RSpm, mC, U, V, mN, N , mSup2, RSup, mSdown2    &
    & , RSdown, mSlepton2, RSlepton, mSneut2, RSneut           &
    & , BRbtosgamma, DeltaMBd, DeltaMBs, BrBToSLL, BtoSNuNu, BR_Bu_TauNu &
    & , a_e, a_mu, a_tau, d_e, d_mu, d_tau                     &
    & , BrMutoEGamma, BrTautoEGamma, BrTautoMuGamma, BrMu3e, BrTau3e, BrTau3Mu &
    & , BR_Z_e_mu, BR_Z_e_tau, BR_Z_mu_tau)

 Else
  BRbtosgamma = 0._dp
  BrBToSLL = 0._dp
  DeltaMBd = 0._dp
  DeltaMBs = 0._dp
  bs_mumu = 0._dp
  BR_Bu_TauNu = 0._dp
  a_e = 0._dp
  a_mu = 0._dp
  a_tau = 0._dp
  d_e = 0._dp
  d_mu = 0._dp
  d_tau = 0._dp
  BrMutoEGamma = 0._dp
  BrTautoEGamma = 0._dp
  BrTautoMuGamma = 0._dp
  BrMu3e = 0._dp
  BrTau3e = 0._dp
  BrTau3Mu = 0._dp
  BR_Z_e_mu = 0._dp
  BR_Z_e_tau = 0._dp
  BR_Z_mu_tau = 0._dp
  rho_parameter = 0._dp
 End If


! If (kont.Ne.0) Then
!  Write(*,*) "there has been a problem, kont = ",kont
! Else

  Call LesHouches_Out(67, 11, kont, HighScaleModel, M_GUT                   &
      & , BRbtosgamma, Bs_MuMu, DeltaMBd, DeltaMBs, BrBToSLL, BtoSNuNu      &
      & , BR_Bu_TauNu  &
      & , a_e, a_mu, a_tau, d_e, d_mu, d_tau, BrMuToEGamma, BrTauToEGamma  &
      & , BrTauToMuGamma, BrMu3e, BrTau3e, BrTau3Mu, BR_Z_e_mu, BR_Z_e_tau &
      & , BR_Z_mu_tau                       &
      & , Rho_parameter, Ecms, Pm, Pp, ISR, SigSup, SigSdown, SigSle      &
      & , SigSn, SigChi0, SigC, SigS0, SigSP, SigHp)
! End If

Call Calculate_Omega_h_sq(omega)

 Call closing() ! closes the files


Contains


 Subroutine CalculateLowEnergyConstraints(gauge, Y_l, Y_d, Y_u, mSpm2, RSpm   &
   & , mC, U, V, mN, N , mSup2, RSup, mSdown2, RSdown, mSlepton2              &
   & , RSlepton, mSneutrino2, RSneutrino, BRbtosgamma, DeltaMBd, DeltaMBs     &
   & , BrBToSLL, BtoSNuNu, BR_Bu_TauNu, a_e, a_mu, a_tau, d_e, d_mu, d_tau    &
   & , BrMutoEGamma, BrTautoEGamma, BrTautoMuGamma, BrMu3e, BrTau3e, BrTau3Mu &
   & , BR_Z_e_mu, BR_Z_e_tau, BR_Z_mu_tau)

 Implicit None
  Real(dp), Intent(in) :: gauge(3), mSpm2(2), mC(2), mN(4), mSneutrino2(3) &
     & , mSup2(6), mSdown2(6), mSlepton2(6)
  Complex(dp), Intent(in), Dimension(3,3) :: Y_d, Y_u, Y_l, RSneutrino 
  Complex(dp), Intent(in) ::  RSpm(2,2), U(2,2), V(2,2), RSup(6,6)   &
     & , RSdown(6,6), RSlepton(6,6), N(4,4)
  Real(dp), Intent(out) :: BRbtosgamma, BrBToSLL, BR_Bu_TauNu, a_mu, a_e      &
   & , a_tau, d_e, d_mu, d_tau, BrMutoEGamma, BrTautoEGamma, BrTautoMuGamma   &
   & , BrMu3e, BrTau3e, BrTau3Mu, BR_Z_e_mu, BR_Z_e_tau, BR_Z_mu_tau, BtoSNuNu
  Complex(dp), Intent(out) :: DeltaMBd, DeltaMBs
  Real(dp) :: MuE_conv_Ti

  Real(dp) :: mudim, dt, tz, g2(213)
  Complex(dp) :: CKMad(3,3), mu_mZ, B_mZ, Mi_mZ(3)
  Complex(dp) :: cpl_CDSu_L(2,3,6), cpl_CDSu_R(2,3,6), cpl_CLSn_L(2,3,3) &
    & , cpl_CLSn_R(2,3,3)
  Integer :: i1,i2,i3, scheme
 Real(dp) :: GMutoEGamma, GTautoEGamma, GTautoMuGamma
 Complex(dp) :: cpl_LNSl_L(3,4,6), cpl_LNSl_R(3,4,6), cpl_NuNSn_R(3,4,3)       &
    & , cpl_CNuSl_L(2,3,6), cpl_CNuSl_R(2,3,6), cpl_NNZ_L(4,4), cpl_NNZ_R(4,4) &
    & , cpl_DGSd_L(3,6), cpl_DGSd_R(3,6), cpl_DNSd_L(3,4,6), cpl_DNSd_R(3,4,6) &
    & , cpl_CCZ_L(2,2), cpl_CCZ_R(2,2), cpl_SnSnZ(3,3), cpl_SlSlZ(6,6)
  Real(dp) :: mf_t(3), BtoSEE, EDM_e(3), EDM_mu(3), EDM_tau(3), gU1, gSU2  &
     & , cosW, sinW2
  Complex(dp) :: c7(7), c7p(6), c8(7), c8p(6)

  Y_l_mZ = Transpose(Y_l) 
  Y_d_mZ = Transpose(Y_d) 
  Y_u_mZ = Transpose(Y_u) 
  A_l_mZ = Transpose(A_l) 
  A_d_mZ = Transpose(A_d) 
  A_u_mZ = Transpose(A_u) 
  M2_E_mZ = M2_E
  M2_L_mZ = M2_L
  M2_D_mZ = M2_D
  M2_U_mZ = M2_U
  M2_Q_mZ = M2_Q
  Mi_mZ = Mi
  mu_mZ = mu
  B_mZ = B
  M2_H_mZ = M2_H

  Call ParametersToG(gauge, y_l_mZ, y_d_mZ, y_u_mZ, Mi_mZ, A_l_mZ, A_d_mZ   &
          & , A_u_mZ, M2_E_mZ, M2_L_mZ, M2_D_mZ, M2_Q_mZ, M2_U_mZ, M2_H_mZ &
          & , mu_mZ, B_mZ, g2)

  mudim = GetRenormalizationScale()
  tz = 0.5_dp * Log(mZ**2/mudim)

  If (tz.Ne.0._dp) Then
   dt = tz / 100._dp
   g2(1) = Sqrt(5._dp / 3._dp ) * g2(1)

   Call odeint(g2, 213, 0._dp, tz, delta_mass, dt, 0._dp, rge213, kont)
   g2(1) = Sqrt(3._dp / 5._dp ) * g2(1)

   Call GToParameters(g2, gauge_mZ, Y_l_mZ, Y_d_mZ, Y_u_mZ, Mi_mZ, A_l_mZ &
                  & , A_d_mZ, A_u_mZ, M2_E_mZ, M2_L_mZ, M2_D_mZ, M2_Q_mZ  &
                  & , M2_U_mZ, M2_H_mZ, mu_mZ, B_mZ)
   Else
    gauge_mZ = gauge
   End If

   Y_l_mZ = Transpose(Y_l_mZ) 
   Y_d_mZ = Transpose(Y_d_mZ) 
   Y_u_mZ = Transpose(Y_u_mZ) 
   A_l_mZ = Transpose(A_l_mZ) 
   A_d_mZ = Transpose(A_d_mZ) 
   A_u_mZ = Transpose(A_u_mZ) 

  !---------------------------------------
  ! BR(b-> s gamma)
  !---------------------------------------
  uD_R = id3C
  uU_R = id3C
  Call Adjungate(CKM, CKMad)
  scheme = GetYukawaScheme()
  If (scheme.Eq.1) Then
   uU_L = CKM
   uD_L = id3C

   If (.Not.GenerationMixing) Y_u_mZ = Matmul(Transpose(CKM),Y_u_mZ)
  Else
   uD_L = CKMad
   uU_L = id3C
   If (.Not.GenerationMixing) Y_d_mZ = Matmul(CKM,Y_d_mZ)
  End If
! I am only interested in the mixing matrices
  Call FermionMass(Y_d_mZ,vevsm(1),mf_t,uD_L,uD_R,kont)
  Call FermionMass(Y_u_mZ,vevsm(2),mf_t,uU_L,uU_R,kont)

  Call BtoQGamma(2, mf_d_mZ, gauge_mZ, mf_u, mW, Y_d_mZ, uD_L, uD_R, Y_u_mZ    &
   & , uU_L, uU_R, mSpm2, RSpm, mC, U, V, mSup2, RSup, A_u_mZ, mSdown2, RSdown &
   & , A_d_mZ, mglu, phaseGlu, mN, N, mu_mZ, mS02, RS0, mP02, RP0, vevSm       &
   & , BRbtosgamma, c7, c7p, c8, c8p)

  Call Delta_MB(1, mf_u, gauge_mZ, Y_u_mZ, uU_L, uU_R, Y_d_mZ, uD_L, uD_R     &
      & , mC, U, V, mN, N, mGlu, phaseGlu, mS02, RS0, mP02, RP0, mSpm2, RSpm  &
      & , mSup2, RSup, A_u_mZ, mu_mZ, mSdown2, RSdown, A_d_mZ, vevSM, DeltaMBd)
  Call Delta_MB(2, mf_u, gauge_mZ, Y_u_mZ, uU_L, uU_R, Y_d_mZ, uD_L, uD_R     &
      & , mC, U, V, mN, N, mGlu, phaseGlu, mS02, RS0, mP02, RP0, mSpm2, RSpm  &
      & , mSup2, RSup, A_u_mZ, mu_mZ, mSdown2, RSdown, A_d_mZ, vevSM, DeltaMBs)
  ! conversion to pico-seconds
  DeltaMBd = 1.e-12_dp*DeltaMBd/hbar
  DeltaMBs = 1.e-12_dp*DeltaMBs/hbar

  Do i1=1,2
   Do i2=1,3
    Do i3=1,3     
     Call CoupCharginoSfermion(i1, i2, i3, gauge_mZ(2), -0.5_dp, RSneut  &
             & , Y_l_mZ, Zero33C, uL_L, uL_R, U, V, cpl_CLSn_L(i1,i2,i3) &
             & , cpl_CLSn_R(i1,i2,i3) )
    End Do
    Do i3=1,6
     Call CoupCharginoSfermion(i1, i2, i3, gauge_mZ(2), 0.5_dp, RSlepton &
      & , Y_l_mZ, Zero33C, uL_L, uL_R, U, V, cpl_CNuSl_L(i1,i2,i3)        &
      & , cpl_CNuSl_R(i1,i2,i3) )
     Call CoupCharginoSfermion(i1, i2, i3, gauge_mZ(2), -0.5_dp, RSup, Y_D_mZ &
       & , Y_U_mZ, id3C, id3C, U, V, cpl_CDSu_L(i1,i2,i3), cpl_CDSu_R(i1,i2,i3))
    End Do
   End Do
  End Do

  Do i1=1,3
   Do i2=1,4
    Do i3=1,6  
     Call CoupNeutralinoSlepton(i1, i2, i3, gauge_mZ(1), gauge_mZ(2), RSlepton &
         & , uL_L, uL_R, Y_l_mZ, N, cpl_LNSl_L(i1,i2,i3), cpl_LNSl_R(i1,i2,i3) )
     Call CoupNeutralinoSdown(i1, i2, i3, gauge_mZ(1), gauge_mZ(2), RSdown &
         & , uD_L, uD_R, Y_d_mZ, N, cpl_DNSd_L(i1,i2,i3), cpl_DNSd_R(i1,i2,i3) )
    End Do
    Do i3=1,3  
     Call CoupNeutralinoSneutrino(i1, i2, i3, gauge_mZ(1), gauge_mZ(2), N &
           & , RSneut, uL_R, cpl_nuNSn_R(i1,i2,i3) )
    End Do
   End Do
  End Do
  Do i1=1,3
    Do i3=1,6  
     Call CoupGluinoSquark3(gauge_mZ(3), phaseGlu, i1, i3, RSdown, uD_L, uD_R &
           & , cpl_DGSd_L(i1,i3), cpl_DGSd_R(i1,i3) )
    End Do
  End Do

  !-------------------------
  ! B_s -> mu+ mu-
  !-------------------------
  Call Bs_to_MuMu(2, mf_u, gauge_mZ, Y_u_mZ, uU_L, uU_R, Y_d_mZ, uD_L, uD_R    &
      & , mC, U, V, mN, N, mGlu, phaseGlu, mS02, RS0, mP02, RP0, mSpm2, RSpm   &
      & , mSup2, RSup, A_u, mu, mSdown2, RSdown, A_d, vevSM, mSneutrino2       &
      & , mSlepton2, cpl_CDSu_L, cpl_CDSu_R, cpl_CLSn_L, cpl_CLSn_R, cpl_LNSl_L&
      & , cpl_LNSl_R, cpl_DGSd_L, cpl_DGSd_R, cpl_DNSd_L, cpl_DNSd_R, Bs_mumu )

  !-------------------------
  ! b -> s l+ l-
  !-------------------------
  Call BToSLL(gauge_mZ, mf_d_mZ, mf_u_mZ, mW, Y_d_mZ, uD_L, uD_R, Y_u_mZ       &
       & , uU_L, uU_R, Y_l_mZ, uL_L, UL_R, mSneut2, Rsneut, mSlepton2, Rslepton&
       & , mSpm2, RSpm, mC, U, V, mSup2, RSup, A_u_mZ, mSdown2, RSdown, A_d_mZ &
       & , mglu, phaseglu, mN, N, mu_mZ, mS02, RS0, mP02, RP0, vevSM           &
       & , BtoSEE, BrBtoSLL)
  !---------------------------------
  ! b -> s nu nu, no QCD corrections
  !---------------------------------
   Call B_To_SNuNu(gauge_mZ, mf_d_mZ, mf_u_mZ, mW, Y_d_mZ, uD_L, uD_R, Y_u_mZ &
        & , uU_L, uU_R, Y_l_mZ, mSneut2, Rsneut, mSlepton2, Rslepton, mSpm2   &
        & , RSpm, mC, U, V, mSup2, RSup, mSdown2, RSdown, mglu, phaseGlu      &
        & , mN, N, vevSM, .False., BtoSNuNu)
  !-------------------
  ! B^-_u -> tau nu
  !-------------------
  BR_Bu_TauNu = Bm_to_l_nu(3,1, mSpm2(2), tanb, RSpm, Y_d_mZ, uU_L &
              &           , uD_R , Y_l_mZ, vevSM)
  !------------------------------------------------------------------
  ! leptonic electric dipole moments
  !------------------------------------------------------------------
  Call Lepton_EDM3(1, mN, mSlepton2, cpl_LNSl_L, cpl_LNSl_R      &
                & , mC, mSneut2, cpl_CLSn_L, cpl_CLSn_R, EDM_e   )
  Call Lepton_EDM3(2, mN, mSlepton2, cpl_LNSl_L, cpl_LNSl_R      &
                & , mC, mSneut2, cpl_CLSn_L, cpl_CLSn_R, EDM_mu  )
  Call Lepton_EDM3(3, mN, mSlepton2, cpl_LNSl_L, cpl_LNSl_R      &
                 & , mC, mSneut2, cpl_CLSn_L, cpl_CLSn_R, EDM_tau )
  d_e = EDM_e(3)
  d_mu = EDM_mu(3)
  d_tau = EDM_tau(3)
  !------------------------------------------------------------------
  ! leptonic anomalous magnetic moments
  !------------------------------------------------------------------
  Call Gminus2(1, mSneut2, mC, cpl_CLSn_L, cpl_CLSn_R, mSlepton2, mN &
             &, cpl_LNSl_L, cpl_LNSl_R, a_e, GenerationMixing)
  Call Gminus2(2, mSneut2, mC, cpl_CLSn_L, cpl_CLSn_R, mSlepton2, mN &
             &, cpl_LNSl_L, cpl_LNSl_R, a_mu, GenerationMixing)
  Call Gminus2(3, mSneut2, mC, cpl_CLSn_L, cpl_CLSn_R, mSlepton2, mN &
             &, cpl_LNSl_L, cpl_LNSl_R, a_tau, GenerationMixing)
  !------------------------------------------------------------------
  ! rare decays of leptons: l -> l' gamma
  !------------------------------------------------------------------
  BrMutoEGamma = 0._dp
  BrTautoEGamma = 0._dp
  BrTautoMuGamma = 0._dp
  If (GenerationMixing) Then
   Call LtoLpGamma(2, 1, mSneut2, mC, cpl_CLSn_L, cpl_CLSn_R, mSlepton2, mN &
                  &, cpl_LNSl_L, cpl_LNSl_R, GMutoEGamma, BrMutoEGamma)
   Call LtoLpGamma(3, 1, mSneut2, mC, cpl_CLSn_L, cpl_CLSn_R, mSlepton2, mN &
                 &, cpl_LNSl_L, cpl_LNSl_R, GTautoEGamma, BrTautoEGamma)
   Call LtoLpGamma(3, 2, mSneut2, mC, cpl_CLSn_L, cpl_CLSn_R, mSlepton2, mN &
                 &, cpl_LNSl_L, cpl_LNSl_R, GTautoMuGamma, BrTautoMuGamma)
  End If
  !------------------------------------------------------------------
  ! rare decays of leptons: l -> 3 l' 
  !------------------------------------------------------------------
  BrMu3E = 0._dp
  BrTau3E = 0._dp
  BrTau3Mu = 0._dp
  gU1 = gauge_mZ(1)
  gSU2 = gauge_mZ(2)
  If (GenerationMixing) Then
   Call BR_lj_to_3li(2, 1, gU1, gSU2, Y_l, mSlepton2, RSlepton, mN, N &
          & , mSneut2, RSneut, mC, U, V, mS02, RS0, mP02, RP0, A_l, mu, vevSM  &
          & , BrMu3e)
   Call BR_lj_to_3li(3, 1, gU1, gSU2, Y_l, mSlepton2, RSlepton, mN, N &
          & , mSneut2, RSneut, mC, U, V, mS02, RS0, mP02, RP0, A_l, mu, vevSM  &
          & , BrTau3e)
   Call BR_lj_to_3li(3, 2, gU1, gSU2, Y_l, mSlepton2, RSlepton, mN, N &
          & , mSneut2, RSneut, mC, U, V, mS02, RS0, mP02, RP0, A_l, mu, vevSM  &
          & , BrTau3Mu)
  End If
  !-------------
  ! delta(rho)
  !-------------
  rho_parameter = DeltaRho(mZ2, mW2, mP02, RP0, mSneutrino2, RSneutrino  &
                &         , mSlepton2, RSlepton, mSup2, RSup, mSdown2    &
                &         , RSdown, mC, U, V, mN, N)
  !----------------------------------
  ! rare Z-boson decays into leptons
  !----------------------------------
  BR_Z_e_mu = 0._dp
  BR_Z_e_tau = 0._dp
  BR_Z_mu_tau = 0._dp

  If (GenerationMixing) Then
   cosW = gSU2 / Sqrt(gU1**2 + gSU2**2)
   sinW2 = gU1**2 / (gU1**2 + gSU2**2)

   cpl_SnSnZ = 0._dp
   Call CoupSneutrinoZ(gSU2, sinW2, cpl_SnSnZ(1,1))
   cpl_SnSnZ(2,2) = cpl_SnSnZ(1,1)
   cpl_SnSnZ(3,3) = cpl_SnSnZ(1,1)

   Do i1=1,6
    Do i2=1,6
     Call CoupSleptonZ(i1, i2, gSU2, sinW2, RSlepton, cpl_SlSlZ(i1,i2))
    End Do
   End Do

   Do i1=1,4
    Do i2=1,4
     Call CoupNeutralinoZ(i1, i2, N, gSU2, cosW &
                       & , cpl_NNZ_L(i1,i2), cpl_NNZ_R(i1,i2))
    End Do
   End Do

   Do i1=1,2
    Do i2=1,2
     Call CoupCharginoZ(i1, i2, U, V, gSU2, cosW &
                     & , cpl_CCZ_L(i1,i2), cpl_CCZ_R(i1,i2))
    End Do
   End Do

   mC2 = mC**2
   mN2 = mN**2

   Call ZtoLiLj(1, 2, .False.  &
       & , mC, mC2, cpl_CLSn_L, cpl_CLSn_R, mSneut2, cpl_SnSnZ &
       & , mN, mN2, cpl_LNSl_L, cpl_LNSl_R, mSlepton2, cpl_SlSlZ &
       & , cpl_CCZ_L, cpl_CCZ_R, cpl_NNZ_L, cpl_NNZ_R        &
       & , BR_Z_e_mu)

   Call ZtoLiLj(1, 3, .True.  &
       & , mC, mC2, cpl_CLSn_L, cpl_CLSn_R, mSneut2, cpl_SnSnZ &
       & , mN, mN2, cpl_LNSl_L, cpl_LNSl_R, mSlepton2, cpl_SlSlZ &
       & , cpl_CCZ_L, cpl_CCZ_R, cpl_NNZ_L, cpl_NNZ_R        &
       & , BR_Z_e_tau)

   Call ZtoLiLj(2, 3, .True.  &
       & , mC, mC2, cpl_CLSn_L, cpl_CLSn_R, mSneut2, cpl_SnSnZ &
       & , mN, mN2, cpl_LNSl_L, cpl_LNSl_R, mSlepton2, cpl_SlSlZ &
       & , cpl_CCZ_L, cpl_CCZ_R, cpl_NNZ_L, cpl_NNZ_R        &
       & , BR_Z_mu_tau)

  End If ! GenerationMixing

  !-----------------------------
  ! this still needs to be done
  !-----------------------------
  MuE_conv_ti = 0._dp

  !-------------------------------------------------------------------------
  ! neutrino masses and mixings, if the dim-5 operator has non-zero entries
  !-------------------------------------------------------------------------
  If (Maxval(Abs(MnuL5)).Gt.0._dp) Then
   MnuL5 = MnuL5 * vevSM(2)**2
   Call NeutrinoMass_1L(MnuL5, gauge(1), gauge(2), Y_l, mC2, U, V, mN2, N &
           & , mSlepton2, Rslepton, mSneutrino2, Rsneutrino, mf_nu, Unu, kont)
  End If

 End Subroutine CalculateLowEnergyConstraints


 Subroutine CalculateSpectrum(n_run, delta, WriteOut, kont, tanb, vevSM     &
     & , mC, U, V, mN, N, mS0, mS02, RS0, mP0, mP02, RP0, mSpm, mSpm2, RSpm &
     & , mSdown, mSdown2, RSdown, mSup, mSup2, RSup, mSlepton, mSlepton2    &
     & , RSlepton, mSneut, mSneut2, RSneut, mGlu, PhaseGlu, gauge           &
     & , uL_L, uL_R, uD_L, uD_R, uU_L, uU_R, Y_l, Y_d, Y_u                  &
     & , Mi, A_l, A_d, A_u, M2_E, M2_L, M2_D, M2_Q, M2_U, M2_H, mu, B       &
     & , m_GUT)
 Implicit None
  Integer, Intent(in) :: n_run
  Logical, Intent(in) :: WriteOut
  Real(dp), Intent(in) :: delta
  Integer, Intent(inout) :: kont
  Real(dp), Intent(inout) :: gauge(3), M2_H(2), tanb, vevSM(2), mP0(2) &
     & , mP02(2), RP0(2,2), mGlu, mSpm(2), mSpm2(2)
  Complex(dp), Intent(inout) :: Y_l(3,3), Y_d(3,3), Y_u(3,3), mu, B, Mi(3)  &
     & , A_l(3,3), A_d(3,3), A_u(3,3), M2_E(3,3), M2_L(3,3), M2_D(3,3)      &
     & , M2_U(3,3), M2_Q(3,3), PhaseGlu, RSpm(2,2)
  Real(dp), Intent(inout) :: mS0(2), mS02(2), RS0(2,2) &
     & , mC(2), mN(4), mSneut(3), mSneut2(3), mSlepton(6), mSlepton2(6) &
     & , mSdown(6), mSdown2(6), mSup(6), mSup2(6), m_GUT
  Complex(dp), Intent(inout) :: U(2,2), V(2,2), N(4,4), Rsneut(3,3)  &
     & , RSlepton(6,6), RSdown(6,6), RSup(6,6)
  Complex(dp), Intent(inout), Dimension(3,3) :: uL_L, uL_R, uD_L, uD_R, uU_L &
     & , uU_R

  Real(dp) :: mZ2_t, mW2_t, Scale_Q, g, gp, dt, tz, g1(57), g7(7), vev      &
       & , mN2(4), mC2(2), g2(213), mglu_T, mP0_T(2), mP02_T(2), RP0_T(2,2)
  Real(dp) :: mC_T(2), mN_T(4), mS0_T(2), mSpm_T(2), mSup_T(6), mSdown_T(6)  &
    & , mSlepton_T(6), mSneut_T(3), mSup2_T(6), mSdown2_T(6), mSlepton2_T(6) &
    & , mSneut2_T(3), mS02_T(2), mSpm2_T(2), RS0_T(2,2), mC2_T(2), mN2_T(4)  &
    & , mass_new(32), mass_old(32), diff_m(32), tanb_in, sinW2, mf3(3)
  Complex(dp) :: U_T(2,2), V_T(2,2), N_T(4,4), RSpm_T(2,2), RSdown_T(6,6)  &
    & , RSup_T(6,6), RSlepton_T(6,6), RSneut_T(3,3), CKM_Q(3,3)
  Integer :: i1, i2, ierr
  Logical :: Converge
  !------------------------------------------------------------------
  ! Performing a first, very rough calculation of the parameters
  ! using 1-loop RGEs and tree-level boundary conditions for gauge and
  ! Yukawa couplings at m_Z for the parameters.
  ! These parameters are used to get a first set of tree-level masses
  ! which are needed for the first of computation of the SUSY thresholds
  ! to gauge and Yukawa couplings. In case of the general MSSM the
  ! parameters are already given.
  !------------------------------------------------------------------
  kont = 0
  If ( (HighScaleModel(1:4).Eq."MSSM").Or.(HighScaleModel.Eq."pMSSM")  &
     & .Or.(HighScaleModel.Eq."NMSSM") )  Then
   sinW2 = 1._dp - mW2/mZ2

   alpha_mZ = Alpha_MSbar(mZ, mW)
   gauge(1) = Sqrt( 4._dp*pi*alpha_mZ/(1._dp-sinW2) )
   gauge(2) = Sqrt( 4._dp*pi*alpha_mZ/sinW2)
   gauge(3) = Sqrt( 4._dp*pi*alphas_mZ)
   gp = gauge(1)
   g = gauge(2)

   vev =  Sqrt( mZ2 * (1._dp - sinW2) * SinW2 / (pi * alpha_mZ) )
   vevSM(1) = vev / Sqrt(1._dp + tanb**2)
   vevSM(2) = tanb * vevSM(1)

   Y_l = 0._dp
   Y_d = 0._dp
   Y_u = 0._dp

   Do i1=1,3
    Y_l(i1,i1) = sqrt2 * mf_l_mZ(i1) / vevSM(1)
    Y_d(i1,i1) = sqrt2 * mf_d_mZ(i1) / vevSM(1)
    Y_u(i1,i1) = sqrt2 * mf_u_mZ(i1) / vevSM(2)
    If (HighScaleModel.Eq."NMSSM") Then
     A_u(i1,i1) = Y_u(i1,i1) * AoY_u(i1,i1)
     A_d(i1,i1) = Y_d(i1,i1) * AoY_d(i1,i1)
     A_l(i1,i1) = Y_l(i1,i1) * AoY_l(i1,i1)
    End If
   End Do
   If (GenerationMixing) Then
    i1 = GetYukawaScheme()
    If (i1.Eq.1) Then
     Y_u = Matmul(Transpose(CKM),Y_u) 
    Else
     Y_d = Matmul(Conjg(CKM),Y_d) 
    End If
   End If

  Else ! high scale model
   Call FirstGuess(phase_mu, tanb, Mi, M2_E, M2_L, A_l, M2_D   &
           & , M2_Q, M2_U, A_d, A_u, mu, B, M2_H, gp, g, Y_l  &
           & , Y_d, Y_u, vevSM, mP02, mP0, kont)
  End If

  If (External_Spectrum) Return ! using the externaly given spectrum

  If (HighScaleModel.Eq."MSSMtree") Then 
   If (GenerationMixing) Then
    Ad_sckm = A_d
    Au_sckm = A_u
    M2D_sckm = M2_D
    M2Q_sckm = M2_Q
    M2U_sckm = M2_U
    Call Switch_from_superCKM(Y_d, Y_u, Ad_sckm, Au_sckm, M2D_sckm, M2Q_sckm &
              & , M2U_sckm, A_d, A_u, M2_D, M2_Q, M2_U, .False. )
   End If
   !------------------------------------------------------------------
   ! last flag implies that Higgs masses at 1-loop effective potential
   !------------------------------------------------------------------
   Call TreeMasses(gp, g, vevSM, Mi(1), Mi(2), Mi(3), mu, B        &
        &, tanb, M2_E, M2_L, A_l, Y_l, M2_D, M2_U, M2_Q, A_d       &
        &, A_u, Y_d, Y_u, mGlu_T, PhaseGlu, mC, mC2, U, V, mN      &
        &, mN2, N, mSneut, mSneut2, Rsneut, mSlepton, mSlepton2    &
        &, RSlepton, mSdown, mSdown2, RSdown, mSup, mSup2, RSup    &
        &, mP0_T, mP02_T, RP0_T, mS0, mS02, RS0, mSpm, mSpm2, RSpm &
        &, GenerationMixing, kont, .True.) 

   tanb_Q = tanb
   mA2_Q = mP02(2)
   vev_Q = Sqrt(vevSM(1)**2 + vevSM(2)**2)
   rho_parameter = 0

  Else If  (HighScaleModel.Eq."NMSSM") Then

   tanb_mZ = tanb
   tanb_Q = tanb
   Call TreeMasses(gp, g, vevSM, vP, Mi(1), Mi(2), Mi(3), mu, B, h0, lam, Ao_h0&
        & , Ao_lam, M2_E, M2_L, A_l, Y_l, M2_D, M2_U, M2_Q, A_d, A_u, Y_d, Y_u &
        & , mGlu, PhaseGlu, mC, mC2, U, V, mN5, mN52, N5                       &
        & , mSneut, mSneut2, Rsneut, mSlepton, mSlepton2, RSlepton             &
        & , mSdown, mSdown2, RSdown, mSup, mSup2, RSup                         &
        & , mP03, mP032, RP03, mS03, mS032, RS03, mSpm, mSpm2, RSpm            &
        & , GenerationMixing, kont, .True.) ! Higgs masses at 1-loop
                                            ! effective potentail
  Else 

   Call TreeMasses(gp, g, vevSM, Mi(1), Mi(2), Mi(3), mu, B        &
        &, tanb, M2_E, M2_L, A_l, Y_l, M2_D, M2_U, M2_Q, A_d       &
        &, A_u, Y_d, Y_u, mGlu_T, PhaseGlu, mC, mC2, U, V, mN      &
        &, mN2, N, mSneut, mSneut2, Rsneut, mSlepton, mSlepton2    &
        &, RSlepton, mSdown, mSdown2, RSdown, mSup, mSup2, RSup    &
        &, mP0_T, mP02_T, RP0_T, mS0, mS02, RS0, mSpm, mSpm2, RSpm &
        &, GenerationMixing, kont, .False.) ! tree-level Higgs mass

   mass_old(1) = Abs(mglu_T)
   mass_old(2:3) = Abs(mC)
   mass_old(4:7) = Abs(mN)
   mass_old(8:9) = mS0
   mass_old(10) = mP0(2)  
   mass_old(11) = mSpm(2)  
   mass_old(12:17) = mSup   
   mass_old(18:23) = mSdown   
   mass_old(24:29) = mSlepton 
   mass_old(30:32) = mSneut
  End If

  if (kont.ne.0) return

   ! In the SPA convention the the renormalization scale is fixed with 1 TeV
  If (SPA_Convention) Call SetRGEScale(1.e3_dp**2)
  Scale_Q = Sqrt(GetRenormalizationScale())

  If (HighScaleModel.Eq."MSSM") Then
  ! MSSM parameters and masses at loop level, all parameters are given at
  ! scale Q, mu and m_A(tree) serve as input in the Higgs sector
   tanb_Q = tanb

   ! calculate first gauge and Yukawa in DR-scheme at m_Z
   kont = 0
   Call BoundaryEW(1, vevSM, mC, U, V, mN, N, mS02, RS0, mP02_T, RP0_T, mSpm  &
    & , mSpm2, RSpm, mSdown, mSdown2, RSdown, mSup, mSup2, RSup            &
    & , mSlepton, mSlepton2, RSlepton, mSneut2, RSneut                     &
    & , uU_L, uU_R, uD_L, uD_R, uL_L, uL_R, mGlu_T, PhaseGlu, mZ2_t, mW2_t &
    & , delta, g1, kont)

   converge = .False.
   Do i1=1,n_run
    tz = Log(Scale_Q/mZ)
    dt = tz / 50._dp
    Call odeint(g1, 57, 0._dp, tz, 0.1_dp*delta, dt, 0._dp, rge57, kont)
    !----------------------------------------------
    ! evolve first parameters up to Q
    !----------------------------------------------
    Call GToCouplings(g1, gauge, Y_l, Y_d, Y_u)
    gauge(1) = Sqrt(3._dp/5._dp) * gauge(1)

    A_l = ZeroC
    A_u = ZeroC
    A_d = ZeroC

    If (GenerationMixing) Then
     Y_l = Transpose(Y_l) 
     Y_d = Transpose(Y_d) 
     Y_u = Transpose(Y_u) 

     A_l = AoY_l * Y_l
     A_d = AoY_d * Y_d
     A_u = AoY_u * Y_u

    
     Call FermionMass(Y_u, sqrt2, mf3, uU_L, uU_R, ierr)
     Call FermionMass(Y_d, sqrt2, mf3, uD_L, uD_R, ierr)
     CKM_Q =  Matmul(uU_L, Transpose(Conjg(ud_L)) )

     M2_D = Matmul( Matmul( Transpose(Conjg(uD_R)), M2D_sckm), uD_R)
     M2_U = Matmul( Matmul( Transpose(Conjg(uU_R)), M2U_sckm), uU_R)
     M2_Q = Matmul( Matmul( Transpose(Conjg(uU_L)), M2Q_sckm), uU_L)

    Else

     Do i2=1,3
      A_l(i2,i2) = AoY_l(i2,i2) * Y_l(i2,i2)
      A_d(i2,i2) = AoY_d(i2,i2) * Y_d(i2,i2)
      A_u(i2,i2) = AoY_u(i2,i2) * Y_u(i2,i2)
     End Do
    End If

    Call LoopMassesMSSM_3(tanb_Q, gauge, Y_l, Y_d, Y_u, Mi, A_l, A_d, A_u &
       & , M2_E, M2_L, M2_D, M2_Q, M2_U, mu, B, 0.1_dp*delta              &
       & , mC, mC2, U, V, mN, mN2, N, mS0, mS02, RS0, mP0, mP02, RP0      &
       & , mSpm, mSpm2, RSpm, mSdown, mSdown2, RSdown, mSup, mSup2        &
       & , RSup, mSlepton, mSlepton2, RSlepton, mSneut, mSneut2           &
       & , RSneut, mGlu, PhaseGlu, M2_H, kont)

    mass_new(1) = Abs(mglu)
    mass_new(2:3) = Abs(mC)
    mass_new(4:7) = Abs(mN)
    mass_new(8:9) = mS0
    mass_new(10) = mP0(2)  
    mass_new(11) = mSpm(2)  
    mass_new(12:17) = mSup   
    mass_new(18:23) = mSdown   
    mass_new(24:29) = mSlepton 
    mass_new(30:32) = mSneut

    diff_m = Abs(mass_new - mass_old)
    Where (mass_old.Ne.0._dp) diff_m = diff_m / mass_old

    If (Maxval(diff_m).Lt.0.1_dp*delta) Then
     converge = .True.
     Exit
    Else
     mass_old = mass_new
    End If

    gauge(1) = Sqrt(5._dp/3._dp) * gauge(1)
    Call ParametersToG(gauge, y_l, y_d, y_u, Mi, A_l, A_d, A_u       &
                  & , M2_E, M2_L, M2_D, M2_Q, M2_U, M2_H, mu, B, g2)

    g7(1:3) = g2(1:3) ! gauge couplings
    g7(4) = g2(20) ! tau Yukawa
    g7(5) = g2(38) ! b Yukawa
    g7(6) = g2(56) ! t Yukawa
    g7(7) = Log( tanb_Q )

    !----------------------------------------------
    ! evolve next tan(beta) down to m_Z
    !----------------------------------------------
    Call odeint(g2, 213, tz, 0._dp, 0.1_dp*delta, dt, 0._dp, rge213, kont)

    Call GToParameters(g2, gauge_mZ, Y_l_mZ, Y_d_mZ, Y_u_mZ, Mi_mZ, A_l_mZ &
       & , A_d_mZ, A_u_mZ, M2_E_mZ, M2_L_mZ, M2_D_mZ, M2_Q_mZ, M2_U_mZ     &
       & , M2_H_mZ, mu_mZ, B_mZ)

    Call odeint(g7, 7, tz, 0._dp, 0.1_dp*delta, dt, 0._dp, rge7, kont)

    tanb_mZ = Exp( g7(7) )

    sinW2 = 1._dp - mW2 / mZ2
    vev =  Sqrt( mZ2 * (1._dp - sinW2) * SinW2 / (pi * alpha_mZ) )
    vevSM(1) = vev / Sqrt(1._dp + tanb_mZ**2)
    vevSM(2) = tanb_mZ * vevSM(1)
    g2(1) = Sqrt(3._dp/5._dp) * g2(1)

    Call TreeMasses(g2(1), g2(2), vevSM, Mi_mZ(1), Mi_mZ(2), Mi_mZ(3), mu_mZ &
      & , B_mZ, tanb_mZ, M2_E_mZ, M2_L_mZ, A_l_mZ, Y_l_mZ, M2_D_mZ, M2_U_mZ  &
      & , M2_Q_mZ, A_d_mZ, A_u_mZ, Y_d_mZ, Y_u_mZ, mGlu_T, PhaseGlu, mC_T    &
      & , mC2_T, U_T, V_T, mN_T, mN2_T, N_T, mSneut_T, mSneut2_T, Rsneut_T   &
      & , mSlepton_T, mSlepton2_T, RSlepton_T, mSdown_T, mSdown2_T, RSdown_T &
      & , mSup_T, mSup2_T, RSup_T, mP0_T, mP02_T, RP0_T, mS0_T, mS02_T       &
      & , RS0_T, mSpm_T, mSpm2_T, RSpm_T, GenerationMixing, kont, .False.)

    If (Min(Minval(mSup2_T), Minval(mSdown2_T), Minval(mSlepton2_T)   &
       &   , Minval(mSneut2_T), Minval(mS02_T), Minval(mP02_T)       &
       &   , Minval(mSpm2_T)).Gt. 0._dp ) Then
     mC = mC_T
     mC2 = mC2_T
     U = U_T
     V = V_T
     mN = mN_T
     mN2 = mN2_T
     N = N_T
     mSneut = mSneut_T
     mSneut2 = mSneut2_T
     Rsneut = Rsneut_T
     mSlepton = mSlepton_T
     mSlepton2 = mSlepton2_T
     RSlepton = RSlepton_T
     mSDown = mSDown_T
     mSDown2 = mSDown2_T
     RSDown = RSDown_T
     mSup = mSup_T
     mSup2 = mSup2_T
     RSup = RSup_T
     mS0 = mS0_T
     mS02 = mS02_T
     RS0 = RS0_T
     mSpm = mSpm_T
     mSpm2 = mSpm2_T
     RSpm = RSpm_T
     YukScen = 1 ! using running masses for boundary conditions at mZ
    Else
     mGlu_T = mGlu
     mP0_T = mP0
     mP02_T = mP02
     RP0_T = RP0
     YukScen = 2 ! using pole masses for boundary conditions at mZ
    End If

    kont = 0
    Call BoundaryEW(i1+1,vevSM, mC, U, V, mN, N, mS02, RS0, mP02_T, RP0_T &
      & , mSpm, mSpm2, RSpm, mSdown, mSdown2, RSdown, mSup, mSup2, RSup   &
      & , mSlepton, mSlepton2, RSlepton, mSneut2, RSneut, uU_L, uU_R      &
      & , uD_L, uD_R , uL_L, uL_R, mGlu_T, PhaseGlu, mZ2_t, mW2_t, delta  &
      & , g1, kont)

   End Do

  Else If (HighScaleModel.Eq."MSSM1") Then
  ! MSSM parameters and masses at loop level, all parameters are given at
  ! scale Q, m^2_(H_i) serve as input in the Higgs sector
   tanb_Q = tanb

   ! calculate first gauge and Yukawa in DR-scheme at m_Z
   kont = 0
   Call BoundaryEW(1, vevSM, mC, U, V, mN, N, mS02, RS0, mP02_T, RP0_T, mSpm  &
    & , mSpm2, RSpm, mSdown, mSdown2, RSdown, mSup, mSup2, RSup            &
    & , mSlepton, mSlepton2, RSlepton, mSneut2, RSneut                     &
    & , uU_L, uU_R, uD_L, uD_R, uL_L, uL_R, mGlu_T, PhaseGlu, mZ2_t, mW2_t &
    & , delta, g1, kont)

   converge = .False.
   tanb_mZ = tanb ! first gues, justified because tanb runs weakly
   Do i1=1,n_run
    !----------------------------------------------
    ! evolve first parameters up to Q
    !----------------------------------------------
    tz = Log(Scale_Q/mZ)
    dt = tz / 50._dp
    Call odeint(g1, 57, -tz, 0._dp, 0.1_dp*delta, -dt, 0._dp, rge57, kont)
    Call GToCouplings(g1, gauge, Y_l, Y_d, Y_u)
    gauge(1) = Sqrt(3._dp/5._dp) * gauge(1)

    If (SPA_Convention) Then
     tanb_in = tanb_Q
    Else
     tanb_in = tanb_mZ
    End If

    A_l = ZeroC
    A_u = ZeroC
    A_d = ZeroC
    Do i2=1,3
     A_l(i2,i2) = AoY_l(i2,i2) * Y_l(i2,i2)
     A_d(i2,i2) = AoY_d(i2,i2) * Y_d(i2,i2)
     A_u(i2,i2) = AoY_u(i2,i2) * Y_u(i2,i2)
    End Do

    Call LoopMassesMSSM(delta, tanb_in, gauge, Y_l, Y_d, Y_u, Mi, A_l, A_d  &
       & , A_u, M2_E, M2_L, M2_D, M2_Q, M2_U, M2_H, phase_mu, mu, B, i1   &
       & , uU_L, uU_R, uD_L, uD_R, uL_L, uL_R                             &
       & , mC, mC2, U, V, mN, mN2, N, mS0, mS02, RS0, mP0, mP02, RP0      &
       & , mSpm, mSpm2, RSpm, mSdown, mSdown2, RSdown, mSup, mSup2        &
       & , RSup, mSlepton, mSlepton2, RSlepton, mSneut, mSneut2           &
       & , RSneut, mGlu, PhaseGlu, kont)

    mass_new(1) = Abs(mglu)
    mass_new(2:3) = Abs(mC)
    mass_new(4:7) = Abs(mN)
    mass_new(8:9) = mS0
    mass_new(10) = mP0(2)  
    mass_new(11) = mSpm(2)  
    mass_new(12:17) = mSup   
    mass_new(18:23) = mSdown   
    mass_new(24:29) = mSlepton 
    mass_new(30:32) = mSneut

    diff_m = Abs(mass_new - mass_old)
    Where (mass_old.Ne.0._dp) diff_m = diff_m / mass_old

    If (Maxval(diff_m).Lt.0.1_dp*delta) Then
     converge = .True.
     Exit
    Else
     mass_old = mass_new
    End If

    gauge(1) = Sqrt(5._dp/3._dp) * gauge(1)
    Call ParametersToG(gauge, y_l, y_d, y_u, Mi, A_l, A_d, A_u       &
                  & , M2_E, M2_L, M2_D, M2_Q, M2_U, M2_H, mu, B, g2)
    !------------------------------
    ! now the running
    !------------------------------
    Call odeint(g2, 213, tz, 0._dp,  0.1_dp*delta, dt, 0._dp, rge213, kont)
    Call GToParameters(g2, gauge_mZ, Y_l_mZ, Y_d_mZ, Y_u_mZ, Mi_mZ, A_l_mZ &
       & , A_d_mZ, A_u_mZ, M2_E_mZ, M2_L_mZ, M2_D_mZ, M2_Q_mZ, M2_U_mZ     &
       & , M2_H_mZ, mu_mZ, B_mZ)
    !-----------------------------------------------------
    ! tanb_mZ is has been calculated in routine LoopMasses
    !-----------------------------------------------------
    sinW2 = 1._dp - mW2 / mZ2
    vev =  Sqrt( mZ2 * (1._dp - sinW2) * SinW2 / (pi * alpha_mZ) )
    vevSM(1) = vev / Sqrt(1._dp + tanb_mZ**2)
    vevSM(2) = tanb_mZ * vevSM(1)
    g2(1) = Sqrt(3._dp/5._dp) * g2(1)
    
    Call TreeMassesMSSM2(g2(1), g2(2), vevSM, Mi_mZ(1), Mi_mZ(2), Mi_mZ(3)   &
      & , mu_mZ, B_mZ , tanb_mZ, M2_E_mZ, M2_L_mZ, A_l_mZ, Y_l_mZ, M2_D_mZ   &
      & , M2_U_mZ, M2_Q_mZ, A_d_mZ, A_u_mZ, Y_d_mZ, Y_u_mZ, uU_L, uU_R       &
      & , uD_L, uD_R, uL_L, uL_R, mGlu_T, PhaseGlu, mC_T, mC2_T, U_T, V_T    &
      & , mN_T, mN2_T, N_T, mSneut_T, mSneut2_T, Rsneut_T, mSlepton_T        &
      & , mSlepton2_T, RSlepton_T, mSdown_T, mSdown2_T, RSdown_T, mSup_T     &
      & , mSup2_T, RSup_T, mP0_T, mP02_T, RP0_T, mS0_T, mS02_T, RS0_T        &
      & , mSpm_T, mSpm2_T, RSpm_T, mZ2_t, mW2_t, GenerationMixing, kont      &
      & , .False., .False.)

    If (Min(Minval(mSup2_T), Minval(mSdown2_T), Minval(mSlepton2_T)  &
       &   , Minval(mSneut2_T), Minval(mS02_T), Minval(mP02_T)       &
       &   , Minval(mSpm2_T)).Gt. 0._dp ) Then
     mC = mC_T
     mC2 = mC2_T
     U = U_T
     V = V_T
     mN = mN_T
     mN2 = mN2_T
     N = N_T
     mSneut = mSneut_T
     mSneut2 = mSneut2_T
     Rsneut = Rsneut_T
     mSlepton = mSlepton_T
     mSlepton2 = mSlepton2_T
     RSlepton = RSlepton_T
     mSDown = mSDown_T
     mSDown2 = mSDown2_T
     RSDown = RSDown_T
     mSup = mSup_T
     mSup2 = mSup2_T
     RSup = RSup_T
     mS0 = mS0_T
     mS02 = mS02_T
     RS0 = RS0_T
     mSpm = mSpm_T
     mSpm2 = mSpm2_T
     RSpm = RSpm_T
     YukScen = 1 ! using running masses for boundary conditions at mZ
    Else
     YukScen = 2 ! using pole masses for boundary conditions at mZ
     mglu_T = mglu
     mP0_T = mP0
     mP02_T = mP02
     RP0_T = RP0     
    End If

    kont = 0
    Call BoundaryEW(i1+1,vevSM, mC, U, V, mN, N, mS02, RS0, mP02_T, RP0_T &
      & , mSpm, mSpm2, RSpm, mSdown, mSdown2, RSdown, mSup, mSup2, RSup   &
      & , mSlepton, mSlepton2, RSlepton, mSneut2, RSneut, uU_L, uU_R      &
      & , uD_L, uD_R , uL_L, uL_R, mGlu_T, PhaseGlu, mZ2_t, mW2_t, delta  &
      & , g1, kont)

   End Do

  Else If (HighScaleModel.Eq."pMSSM") Then
   ! calculate first gauge and Yukawa in DR-scheme at m_Z
   Call BoundaryEW(1, vevSM, mC, U, V, mN, N, mS02, RS0, mP02_T, RP0_T, mSpm  &
    & , mSpm2, RSpm, mSdown, mSdown2, RSdown, mSup, mSup2, RSup            &
    & , mSlepton, mSlepton2, RSlepton, mSneut2, RSneut                     &
    & , uU_L, uU_R, uD_L, uD_R, uL_L, uL_R, mGlu_T, PhaseGlu, mZ2_t, mW2_t &
    & , delta, g1, kont)

   converge = .False.
   mP0(1) = mZ
   mP02(1) = mZ2
   Do i1=1,n_run
    tz = Log(Scale_Q/mZ)
    dt = tz / 50._dp
    kont = 0
    Call odeint(g1, 57, 0._dp, tz,  0.1_dp*delta, dt, 0._dp, rge57, kont)

    !----------------------------------------------
    ! evolve first parameters up to Q
    !----------------------------------------------
    Call GToCouplings(g1, gauge, Y_l, Y_d, Y_u)
    gauge(1) = Sqrt(3._dp/5._dp) * gauge(1)

    If (GenerationMixing) then
     Y_l = Transpose(Y_l) 
     Y_d = Transpose(Y_d) 
     Y_u = Transpose(Y_u) 
     
     Call Switch_from_superCKM(Y_d, Y_u, Ad_sckm, Au_sckm, M2D_sckm, M2Q_sckm &
               & , M2U_sckm, A_d, A_u, M2_D, M2_Q, M2_U, .False. )
    Else ! .not. GenerationMixing
     A_l = ZeroC
     A_u = ZeroC
     A_d = ZeroC
     Do i2=1,3
      A_l(i2,i2) = AoY_l(i2,i2) * Y_l(i2,i2)
      A_d(i2,i2) = AoY_d(i2,i2) * Y_d(i2,i2)
      A_u(i2,i2) = AoY_u(i2,i2) * Y_u(i2,i2)
     End Do
    End If

    kont = 0
    Call LoopMassesMSSM_2(delta, tanb, gauge, Y_l, Y_d, Y_u, Mi, A_l, A_d &
       & , A_u, M2_E, M2_L, M2_D, M2_Q, M2_U, mu                          &
       & , mC, mC2, U, V, mN, mN2, N, mS0, mS02, RS0, mP02, RP0           &
       & , mSpm, mSpm2, RSpm, mSdown, mSdown2, RSdown, mSup, mSup2        &
       & , RSup, mSlepton, mSlepton2, RSlepton, mSneut, mSneut2           &
       & , RSneut, mGlu, PhaseGlu, M2_H, B, kont)

    mass_new(1) = Abs(mglu)
    mass_new(2:3) = Abs(mC)
    mass_new(4:7) = Abs(mN)
    mass_new(8:9) = mS0
    mass_new(10) = mP0(2)  
    mass_new(11) = mSpm(2)  
    mass_new(12:17) = mSup   
    mass_new(18:23) = mSdown   
    mass_new(24:29) = mSlepton 
    mass_new(30:32) = mSneut

    diff_m = Abs(mass_new - mass_old)
    Where (mass_old.Ne.0._dp) diff_m = diff_m / mass_old

    If (Maxval(diff_m).Lt.delta) Then
     converge = .True.
     Exit
    Else
     mass_old = mass_new
    End If

    If (i1.Eq.50) Then
     Write(*,*) "Problem with accuracy (pMSSM)",diff_m,delta,i1
     Exit
    End If

    gauge(1) = Sqrt(5._dp/3._dp) * gauge(1)

    Y_l = Transpose(Y_l) 
    Y_d = Transpose(Y_d) 
    Y_u = Transpose(Y_u) 
    A_l = Transpose(A_l) 
    A_d = Transpose(A_d) 
    a_u = Transpose(A_u) 
  
   Call ParametersToG(gauge, y_l, y_d, y_u, Mi, A_l, A_d, A_u       &
                  & , M2_E, M2_L, M2_D, M2_Q, M2_U, M2_H, mu, B, g2)
    Call odeint(g2, 213, tz, 0._dp, 0.1_dp*delta, dt, 0._dp, rge213, kont)

    Call GToParameters(g2, gauge_mZ, Y_l_mZ, Y_d_mZ, Y_u_mZ, Mi_mZ, A_l_mZ &
       & , A_d_mZ, A_u_mZ, M2_E_mZ, M2_L_mZ, M2_D_mZ, M2_Q_mZ, M2_U_mZ     &
       & , M2_H_mZ, mu_mZ, B_mZ)

    Y_l_mZ = Transpose(Y_l_mZ) 
    Y_d_mZ = Transpose(Y_d_mZ) 
    Y_u_mZ = Transpose(Y_u_mZ) 
    A_l_mZ = Transpose(A_l_mZ) 
    A_d_mZ = Transpose(A_d_mZ) 
    a_u_mZ = Transpose(A_u_mZ)
 
    sinW2 = 1._dp - mW2 / mZ2
    vev =  Sqrt( mZ2 * (1._dp - sinW2) * SinW2 / (pi * alpha_mZ) )
    vevSM(1) = vev / Sqrt(1._dp + tanb**2)
    vevSM(2) = tanb * vevSM(1)
    g2(1) = Sqrt(3._dp/5._dp) * g2(1)
    
    Call TreeMasses(g2(1), g2(2), vevSM, Mi_mZ(1), Mi_mZ(2), Mi_mZ(3), mu_mZ  &
      & , B_mZ , tanb, M2_E_mZ, M2_L_mZ, A_l_mZ, Y_l_mZ, M2_D_mZ, M2_U_mZ     &
      & , M2_Q_mZ, A_d_mZ, A_u_mZ, Y_d_mZ, Y_u_mZ, mGlu_T, PhaseGlu, mC_T     &
      & , mC2_T, U_T, V_T, mN_T, mN2_T, N_T, mSneut_T, mSneut2_T, Rsneut_T    &
      & , mSlepton_T, mSlepton2_T, RSlepton_T, mSdown_T, mSdown2_T, RSdown_T  &
      & , mSup_T, mSup2_T, RSup_T, mP0_T, mP02_T, RP0_T, mS0_T, mS02_T, RS0_T &
      & , mSpm_T, mSpm2_T, RSpm_T, GenerationMixing, kont, .False.)

    If (Min(Minval(mSup2_T), Minval(mSdown2_T), Minval(mSlepton2_T)   &
       &   , Minval(mSneut2_T), Minval(mS02_T), Minval(mP02_T)       &
       &   , Minval(mSpm2_T)).Gt. 0._dp ) Then
     mC = mC_T
     mC2 = mC2_T
     U = U_T
     V = V_T
     mN = mN_T
     mN2 = mN2_T
     N = N_T
     mSneut = mSneut_T
     mSneut2 = mSneut2_T
     Rsneut = Rsneut_T
     mSlepton = mSlepton_T
     mSlepton2 = mSlepton2_T
     RSlepton = RSlepton_T
     mSDown = mSDown_T
     mSDown2 = mSDown2_T
     RSDown = RSDown_T
     mSup = mSup_T
     mSup2 = mSup2_T
     RSup = RSup_T
     mS0 = mS0_T
     mS02 = mS02_T
     RS0 = RS0_T
     mSpm = mSpm_T
     mSpm2 = mSpm2_T
     RSpm = RSpm_T
     YukScen = 1 ! using running masses for boundary conditions at mZ
    Else
     YukScen = 2 ! using pole masses for boundary conditions at mZ
     mglu_T = mglu
     mP0_T = mP0
     mP02_T = mP02
     RP0_T = RP0     
    End If

    kont = 0
    Call BoundaryEW(i1+1,vevSM, mC, U, V, mN, N, mS02, RS0, mP02_T, RP0_T &
      & , mSpm, mSpm2, RSpm, mSdown, mSdown2, RSdown, mSup, mSup2, RSup   &
      & , mSlepton, mSlepton2, RSlepton, mSneut2, RSneut, uU_L, uU_R      &
      & , uD_L, uD_R , uL_L, uL_R, mGlu_T, PhaseGlu, mZ2_t, mW2_t, delta  &
      & , g1, kont)

   End Do

  Else If (HighScaleModel.Eq."MSSMtree") Then
   mP0 = mP0_T
   mP02 = mP02_T
   RP0 = RP0_T
   mglu = mGlu_T
   If (GenerationMixing) Then

    Call FermionMass(Y_l,vevSM(1),mf_l_mZ,uL_L,uL_R,kont)
    Call FermionMass(Y_d,vevSM(1),mf_d_mZ,uD_L,uD_R,kont)
    Call FermionMass(Y_u,vevSM(2),mf_u_mZ,uU_L,uU_R,kont)

   Else
    uL_L = id3C
    uL_R = id3C
    uD_L = id3C
    uD_R = id3C
    uU_L = id3C
    uU_R = id3C
   End If
   
  Else If (HighScaleModel.Eq."NMSSM") Then
   If (GenerationMixing) Then
    Call FermionMass(Y_l,vevSM(1),mf_l,uL_L,uL_R,kont)
    Call FermionMass(Y_d,vevSM(1),mf_d,uD_L,uD_R,kont)
    Call FermionMass(Y_u,vevSM(2),mf_u,uU_L,uU_R,kont)
   Else
    uL_L = id3C
    uL_R = id3C
    uD_L = id3C
    uD_R = id3C
    uU_L = id3C
    uU_R = id3C
   End If
   
  Else ! high scale models
   mP0 = mP0_T
   mP02 = mP02_T
   RP0 = RP0_T
   mglu = mGlu_T
  !-------------------------------------------------------------------------
  ! Iterative procedure to get the 1-loop masses and mixing matrices;
  ! delta is the relative precision required for the masses after two runs
  ! The procedure is stopped after the i-th iteration 
  ! if delta > |m(i) - m(i-1)|/m(i) for all SUSY. In the case that more than
  ! 20 iterations are necessary the iteration loop is left and a warning is 
  ! given.
  !--------------------------------------------------------------------------
   ! In the SPA convention the the renormalization scale is fixed with 1 TeV
   If (SPA_Convention) Call SetRGEScale(1.e3_dp**2) ! 

   Call Sugra(delta, vevSM, mC, U, V, mN, N, mS0, mS02, RS0 &
     & , mP0, mP02, RP0, mSpm, mSpm2, RSpm, mSdown, mSdown2 &
     & , RSdown, mSup, mSup2, RSup, mSlepton, mSlepton2     &
     & , RSlepton, mSneut, mSneut2, RSneut, mGlu, PhaseGlu  &
     & , gauge, uL_L, uL_R, uD_L, uD_R, uU_L, uU_R, Y_l     &
     & , Y_d, Y_u, Mi, A_l, A_d, A_u, M2_E, M2_L, M2_D      &
     & , M2_Q, M2_U, M2_H, mu, B, m_GUT, kont, WriteOut, n_run)
  End If

 End Subroutine CalculateSpectrum


 Subroutine ReadingData(kont)
 !--------------------------------------------------------
 ! reading the input, all routines used can be found in
 ! InputOutput.f90 
 !--------------------------------------------------------
 Implicit None
  Integer, Intent(out) :: kont

  Logical :: file_exists

  kont = -123456

 !------------------------------------------------------------------
 ! Checked, if the file LesHouches.in exists
 !------------------------------------------------------------------
  Inquire(file="LesHouches.in",exist=file_exists)

 !---------------------------------------
 !   if yes, use the data from this file
 !---------------------------------------
  If (file_exists) Then
   kont = 1
   Call LesHouches_Input(kont, HighScaleModel, Ecms, Pm, Pp, ISR, F_GMSB)
   LesHouches_Format = .True.
Write(*,*) ckm(1,3)
  Else
   LesHouches_Format = .False.
   !-----------------------------------
   ! perform the original squence
   !-----------------------------------
   !--------------------------------------------------------------------------
   ! Intialisation routines
   ! InitialzeControl: initialze the control system and opens the output file
   !           can be influenced using the file Control.in 
   ! InitializeLoopFunctions: calculates the constants needed for the 
   !            calculation of the loop functions
   ! InitializeStandardModel: sets the Standard Model parameters; in the case
   !            that the file StandardModel.in is present the data will be read
   !            in, otherwise default values will be used as described in the
   !            manual
   !--------------------------------------------------------------------------
   Call InitializeControl(11, "SPheno3.out", "SPheno3")
   Call InitializeLoopFunctions
   Call InitializeStandardModel
   If (l_CS) Call InitializeCrossSections(Ecms, Pm, Pp, ISR)
   !------------------------------------------------------------------
   ! reading the data for the specification of the high scale model,
   ! source code of the subroutine is in the module InputOutput
   !------------------------------------------------------------------
   Inquire(file="HighScale.in",exist=file_exists)

   If (file_exists) Then
    kont = 0
    Call HighScaleInput()
   Else
    Write(*,*) &
     & "Neither the file 'HighScale.in' nor the file 'LesHouches.in'"
    Write(*,*) "has been found. Please provide an input file."
    Call TerminateProgram 
   End If
  End If

 End Subroutine ReadingData


 subroutine Calculate_Omega_h_sq(omega)
 implicit none
  Real(dp), intent(out) :: omega

  integer :: kont, n_run, i1, ierr
   Real(dp) :: g, gp, delta
  !------------------------------
  ! masses and mixing angles
  !------------------------------
  Complex(dp), Dimension(3,3) :: uL_L, uL_R, uD_L, uD_R, uU_L, uU_R
  Logical :: WriteOut, check

  Integer, Save  :: f_c=1, k1, k2, k3, k4, io_L=65
  Character(len=11) :: f_n
  logical :: non_minimal
  integer :: i3
  Real(dp) :: Yu(3), Yd(3), Q, MaxCont, mat6r(6,6), nr(4,4), mnr(4)
  Complex(dp), Dimension(3,3) :: CKM_Q
  Complex(dp), Dimension(6,6) :: RUsq_ckm, RDsq_ckm
  Character(len=10) :: name, c_m, c_snu(3), c_sle(6)     &
      & , c_su(6), c_sd(6), c_slep(6), c_grav, zeit
  Character(len=6) :: c_lm(3), c_lp(3), c_nu(3), c_gl, c_cp(5) &
     & , c_cm(5), c_c0(8), c_phot, c_sp(7), c_sm(7)
  Character(len=1) :: c_u(3), c_d(3)
  Character(len=4) :: c_S0(6), c_P0(5)
  Character(len=8) :: Datum
  Character(len=13) :: c_sfermion 
  Integer :: n_n, n_c, n_s0, n_p0, n_spm, n_sl, n_sn, n_sd, n_su 
  Integer :: id_n(8), id_sp(7), id_sm(7), id_S0(6), id_P0(5), id_c(5) &
      & , id_snu(3), i_zaehl, id_sle(6), id_f, id_fp, id_su(6), ii, jj &
      & , id_sd(6)
  Integer, Parameter ::id_A0 = 36, id_Hp = 37                            &
      & , id_W = 24, id_Z = 23, id_ph = 22, id_gl = 21                   &
      & , id_l(3) = (/ 11, 13, 15 /), id_nu(3) = (/ 12, 14, 16 /)        &
      & , id_u(3) = (/ 2, 4, 6 /), id_d(3) = (/ 1, 3, 5 /)               &
      & , id_grav = 1000039, id_glu = 1000021
  logical :: use_flavour_states=.true.
  Real(dp) :: T3, YL, YR, ML, MR, mSf(2), mSf2(2)
  Complex(dp) :: Rsf(2,2), A, Y

  omega=0._dp

  Iname = Iname + 1
  NameOfUnit(Iname) = "Calculate_Omega_h_sq"


  c_phot = "photon"
  c_grav = "~Gravitino"
  c_lm(1) = "e-"
  c_lp(1) = "e+"
  c_lm(2) = "mu-"
  c_lp(2) = "mu+"
  c_lm(3) = "tau-"
  c_lp(3) = "tau+"

  If (GenerationMixing) Then
   c_nu(1) = "nu_1"
   c_nu(2) = "nu_2"
   c_nu(3) = "nu_3"
  Else
   c_nu(1) = "nu_e"
   c_nu(2) = "nu_mu"
   c_nu(3) = "nu_tau"
  End If
  c_d(1) = "d"
  c_d(2) = "s"
  c_d(3) = "b"
  c_u(1) = "u"
  c_u(2) = "c"
  c_u(3) = "t"
  c_gl = "~g"
  c_cp(1) = "chi_1+"
  c_cp(2) = "chi_2+"
  c_cm(1) = "chi_1-"
  c_cm(2) = "chi_2-"
  c_c0(1) = "chi_10"
  c_c0(2) = "chi_20"
  c_c0(3) = "chi_30"
  c_c0(4) = "chi_40"
  id_n(1) = 1000022
  id_n(2) = 1000023
  id_n(3) = 1000025
  id_n(4) = 1000035

  c_sp(1) = "H+"
  c_sm(1) = "H-"
  id_sp = 0
  id_sp(1) = 37
  id_sm = - id_sp  

  n_sd = 6
  n_su = 6
 
  id_c(1) = 1000024
  id_c(2) = 1000037

   c_s0(1) = "h0"
   c_s0(2) = "H0"
   id_S0(1) = 25
   id_S0(2) = 35
   c_P0(1) = "A0"
   id_P0(1) = 36
   n_n = 4
   n_c = 2
   n_s0 = 2
   n_p0 = 1 
   n_spm = 1 
   n_sl = 6
   n_sn = 3

  Q = Sqrt( GetRenormalizationScale() )
  gp = gauge(1)
  g = gauge(2)
  Open(65,file="SPheno_1.spc",status="unknown")
  !--------------------------------------------------------
  ! General information
  !--------------------------------------------------------
  ! Les Houches standard
  !-----------------------
   Write(io_L,100) "# SUSY Les Houches Accord 2.beta - MSSM spectrum + Decays"
   Write(io_L,100) "# SPheno 3.xx beta"
   Write(io_L,100) &
     & "# W. Porod, Comput. Phys. Commun. 153 (2003) 275-315, hep-ph/0301101"
   Write(io_L,100) "# in case of problems send email to porod@ific.uv.es"
   Write(io_L,100) "# Created: today"
   Write(io_L,100) "Block SPINFO         # Program information"
   Write(io_L,100) "     1   SPheno      # spectrum calculator"
   Write(io_L,100) "     2   3.xx beta   # version number"

   !--------------------------------------
   ! model information
   !--------------------------------------
   If (HighScaleModel.Eq."mSugra") Then
    non_minimal = .False.
    Write(io_L,100) "Block MODSEL  # Model selection"
    Write(io_L,100) "    1    1    # mSUGRA model"
    If (GenerationMixing) Write(io_L,100) &
      & " 6 1                      # switching on flavour violation"
    Write(io_L,100) "Block MINPAR  # Input parameters"
    Write(io_L,101) 1,Sqrt(Real(M2_E_0(1,1),dp)),"# m0      "
    Write(io_L,101) 2,Real(Mi_0(1),dp),"# m12     "
    Write(io_L,101) 3,tanb_mZ,"# tanb at m_Z   "
    Write(io_L,101) 4, Real(phase_mu,dp),"# Sign(mu)"
    Write(io_L,101) 5,Real(AoY_l_0(1,1),dp),"# A0"
    Write(io_L,100) "#"

    Write(io_L,106) "Block gauge Q=",m_GUT,"# (GUT scale)"
    Write(io_L,104) 1,gauge_0(1),"# g'(Q)^DRbar"
    Write(io_L,104) 2,gauge_0(2),"# g(Q)^DRbar"
    Write(io_L,104) 3,gauge_0(3),"# g3(Q)^DRbar"
    
   Else If ((HighScaleModel(1:5).Eq."SUGRA").Or.   &
          & (HighScaleModel(1:5).Eq."SEESA")) Then
    non_minimal = .False.
    Write(io_L,100) "Block MODSEL  # Model selection"
    Write(io_L,100) "    1    1    # mSUGRA model"
    If (GenerationMixing) Write(io_L,100) &
      & " 6 1                      # switching on flavour violation"
    If (HighScaleModel.Eq."SUGRA_SU5") Then
     Write(io_L,100) "    3    2    # mSUGRA model + SU(5)"
    Else If (HighScaleModel.Eq."SUGRA_NuR1") Then
     Write(io_L,100) "    3    3    # mSUGRA model + nu_R at a common scale"
     Write(io_L,100) "Block MnuR  # mass scale of the right handed neutrinos"
     Write(io_L,101) 1,MnuR(1),"# m_nu_R      "     
    Else If (HighScaleModel.Eq."SUGRA_NuR") Then
     Y_nu_0 = Transpose(Y_nu_0) ! in the RGEs the transposed Yukawas are used
     Write(io_L,100) "    3    4    # mSUGRA model + three  nu_R, ~nu_R"
     Write(io_L,106) "Block Ynu0 Q=",m_GUT,"# (GUT scale)"
     Do i1=1,3
      Do i2=1,3
       Write(io_L,105) i2,i1,Real(Y_nu_0(i2,i1),dp), &
             & "# Y_(nu,"//bu(i2)//bu(i1)//")"  
      End Do
     End Do
     If (Maxval(Abs(Aimag(Y_nu_0))).Gt.0._dp) Then
      Write(io_L,106) "Block IMYnu0 Q=",m_GUT,"# (GUT scale)"
      Do i1=1,3
       Do i2=1,3
        Write(io_L,105) i2,i1,Aimag(Y_nu_0(i2,i1)) &
             & ,"# Im(Y_(nu,"//bu(i2)//bu(i1)//"))"  
       End Do
      End Do
     End If
    Else If (HighScaleModel.Eq."SEESAW_II") Then
     Write(io_L,100) "    3    5    # mSUGRA model + Higgs triplett"
     Write(io_L,106) "Block YT0 Q=",m_GUT,"# (GUT scale)"
     Do i1=1,3
      Do i2=i1,3
       Write(io_L,105) i2,i1,Real(Y_T_0(i2,i1),dp),"# Y_(T,"//bu(i2)//bu(i1)//")"  
      End Do
     End Do
     If (Maxval(Abs(Aimag(Y_T_0))).Gt.0._dp) Then
      Write(io_L,106) "Block IMYT0 Q=",m_GUT,"# (GUT scale)"
      Do i1=1,3
       Do i2=i1,3
        Write(io_L,105) i2,i1,Aimag(Y_T_0(i2,i1)) &
             & ,"# Im(Y_(T,"//bu(i2)//bu(i1)//"))"  
       End Do
      End Do
     End If
     Write(io_L,106) "Block Higgs3 Q=",m_GUT,"# (GUT scale)"
     Write(io_L,101) 1,M_H3(1),"# m_H3      "
     Write(io_L,101) 2,Real(lam12_0(1),dp),"# Re(lambda_1)"
     If (Aimag(lam12_0(1)).Ne.0._dp) &
         Write(io_L,101) 3,Aimag(lam12_0(1)),"# Im(lambda_1)"
     Write(io_L,101) 4,Real(lam12_0(2),dp),"# Re(lambda_2)"
     If (Aimag(lam12_0(2)).Ne.0._dp) &
         Write(io_L,101) 5,Aimag(lam12_0(2)),"# Im(lambda_2)"
     If (Fifteen_plet) Then
      Write(io_L,100) "    6    1               # using RGEs for 15-plet"
     Else
      Write(io_L,100) "    6    0               # using RGEs for 3-plet"
     End If
      
    End If
    Write(io_L,100) "Block MINPAR  # Input parameters"
    Write(io_L,101) 1,Sqrt(Real(M2_E_0(1,1),dp)),"# m0      "
    Write(io_L,101) 2,Real(Mi_0(1),dp),"# m12     "
    Write(io_L,101) 3,tanb_mZ,"# tanb at m_Z   "
    Write(io_L,101) 4, Real(phase_mu,dp),"# Sign(mu)"
    Write(io_L,101) 5,Real(AoY_l_0(1,1),dp),"# A0"
    If (HighScaleModel.Eq."SUGRA_SU5") Then
     Write(io_L,101) 7,Max(M_SO_10,m_GUT),"# SO(10) scale"
     Write(io_L,101) 8,D_SO_10,"# D-terms at SO(10) scale"
    End If
    Write(io_L,100) "#"

    Write(io_L,106) "Block gauge Q=",m_GUT,"# (GUT scale)"
    Write(io_L,104) 1,gauge_0(1),"# g'(Q)^DRbar"
    Write(io_L,104) 2,gauge_0(2),"# g(Q)^DRbar"
    Write(io_L,104) 3,gauge_0(3),"# g3(Q)^DRbar"
    
   Else If (HighScaleModel.Eq."AMSB") Then
    non_minimal = .False.
    Write(io_L,100) "Block MODSEL  # Model selection"
    Write(io_L,100) "    1    3    # mAMSB model"
    If (GenerationMixing) Write(io_L,100) &
      & " 6 1                      # switching on flavour violation"
    Write(io_L,100) "Block MINPAR  # Input parameters"
    Write(io_L,101) 1,m0_amsb,"# M_0" 
    Write(io_L,101) 2,m_32,"# m_3/2, gravitino mass"
    Write(io_L,101) 3,tanb_mZ,"# tanb at m_Z   "
    Write(io_L,101) 4, Real(phase_mu,dp),"# Sign(mu)"

    
   Else
    non_minimal = .True.
    Write(io_L,100) "# Either the general MSSM or a model has been used"
    Write(io_L,100) &
      & "# which has not yet been implemented in the LesHouches standard"
    Write(io_L,100) "Block MINPAR  # Input parameters"
    Write(io_L,101) 3,tanb_mZ,"# tanb at m_Z    "
    Write(io_L,101) 4, Real(phase_mu,dp),"# Sign(mu)"
   End If 

   !---------------------------------------------------
   ! parameters + masses for SPheno.spc
   !---------------------------------------------------
   Write(io_L,100) "Block SMINPUTS  # SM parameters"
   Write(io_L,102) 1, 1._dp / alpha_MSbar(mZ, mW),"# alpha_em^-1(MZ)^MSbar"
   Write(io_L,102) 2,G_F,"# G_mu [GeV^-2]"
   Write(io_L,102) 3,alphaS_MZ,"# alpha_s(MZ)^MSbar"
   Write(io_L,102) 4,mZ,"# m_Z(pole)"
   Write(io_L,102) 5,mf_d(3),"# m_b(m_b), MSbar"
   Write(io_L,102) 6,mf_u(3),"# m_t(pole)"
   Write(io_L,102) 7,mf_l(3),"# m_tau(pole)"

   !----------------------------------------------------------------
   ! in the case of GenerationMixing all parameters and mixings
   ! are given in the SuperCKM basis for squarks
   !----------------------------------------------------------------
   If (GenerationMixing) Then
    Write(io_L,100) "Block VCKMIN  # CKM matrix, Wolfenstein parameterization"
    Write(io_L,102) 1, lam_wolf,"# lambda"
    Write(io_L,102) 2,A_wolf,"# A"
    Write(io_L,102) 3,rho_wolf,"# rho bar"
    Write(io_L,102) 4,eta_wolf,"# eta bar"
    Call Switch_to_superCKM(Y_d, Y_u, A_d, A_u, M2_D, M2_Q, M2_U         &
              &, Ad_sckm, Au_sckm, M2D_sckm, M2Q_sckm, M2U_sckm, .False. &
              &, RSdown, RSup, Rdsq_ckm, RUsq_ckm, CKM_Q, Yd, Yu )

   Else ! .non.GenerationMixing

    Do i1=1,3
     Yu(i1) = Real(Y_u(i1,i1),dp)
     Yd(i1) = Real(Y_d(i1,i1),dp)
    End Do
    Ad_sckm = A_d
    Au_sckm = A_u

    M2D_SCKM = M2_D
    M2U_SCKM = M2_U
    M2Q_SCKM = M2_Q

    RUsq_ckm = RSup
    RDsq_ckm = RSdown

   End If

   If (non_minimal) Then
    Write(io_L,100) "Block EXTPAR  # "
    Write(io_L,104) 0,Q , "# scale Q where the parameters below are defined"
    Write(io_L,104) 1,Real(Mi(1),dp),"# M_1"
    Write(io_L,104) 2,Real(Mi(2),dp),"# M_2"
    Write(io_L,104) 3,Real(Mi(3),dp),"# M_3"
    Write(io_L,104) 11,Real(Au_sckm(3,3)/y_u(3,3),dp), "# A_t"
    Write(io_L,104) 12,Real(Ad_sckm(3,3)/y_d(3,3),dp), "# A_b"
    Write(io_L,104) 13,Real(A_l(3,3)/y_l(3,3),dp), "# A_l"
    Write(io_L,104) 23,Real(mu ,dp), "# mu "
    Write(io_L,104) 31,Sqrt(Real(M2_L(1,1),dp)),"# M_(L,11)"
    Write(io_L,104) 32,Sqrt(Real(M2_L(2,2),dp)),"# M_(L,22)"
    Write(io_L,104) 33,Sqrt(Real(M2_L(3,3),dp)),"# M_(L,33)"
    Write(io_L,104) 34,Sqrt(Real(M2_E(1,1),dp)),"# M_(E,11)"
    Write(io_L,104) 35,Sqrt(Real(M2_E(2,2),dp)),"# M_(E,22)"
    Write(io_L,104) 36,Sqrt(Real(M2_E(3,3),dp)),"# M_(E,33)"
    Write(io_L,104) 41,Sqrt(Real(M2Q_SCKM(1,1),dp)),"# M_(Q,11)"
    Write(io_L,104) 42,Sqrt(Real(M2Q_SCKM(2,2),dp)),"# M_(Q,22)"
    Write(io_L,104) 43,Sqrt(Real(M2Q_SCKM(3,3),dp)),"# M_(Q,33)"
    Write(io_L,104) 44,Sqrt(Real(M2U_SCKM(1,1),dp)),"# M_(U,11)"
    Write(io_L,104) 45,Sqrt(Real(M2U_SCKM(2,2),dp)),"# M_(U,22)"
    Write(io_L,104) 46,Sqrt(Real(M2U_SCKM(3,3),dp)),"# M_(U,33)"
    Write(io_L,104) 47,Sqrt(Real(M2D_SCKM(1,1),dp)),"# M_(D,11)"
    Write(io_L,104) 48,Sqrt(Real(M2D_SCKM(2,2),dp)),"# M_(D,22)"
    Write(io_L,104) 49,Sqrt(Real(M2D_SCKM(3,3),dp)),"# M_(D,33)"
   End If
      
! couplings
  Write(io_L,106) "Block gauge Q=",Q,"# (SUSY scale)"
  Write(io_L,104) 1,gauge(1),"# g'(Q)^DRbar"
  Write(io_L,104) 2,gauge(2),"# g(Q)^DRbar"
  Write(io_L,104) 3,gauge(3),"# g3(Q)^DRbar"

  Write(io_L,106) "Block Yu Q=",Q,"# (SUSY scale)"
  Write(io_L,107) 1,1,Yu(1), "# Y_u(Q)^DRbar"
  Write(io_L,107) 2,2,Yu(2), "# Y_c(Q)^DRbar"
  Write(io_L,107) 3,3,Yu(3), "# Y_t(Q)^DRbar"

  Write(io_L,106) "Block Yd Q=",Q,"# (SUSY scale)"
  Write(io_L,107) 1,1,Yd(1), "# Y_d(Q)^DRbar"
  Write(io_L,107) 2,2,Yd(2), "# Y_s(Q)^DRbar"
  Write(io_L,107) 3,3,Yd(3), "# Y_b(Q)^DRbar"

  If (GenerationMixing) Then 
                             
   Write(io_L,106) "Block VCKM Q=",Q,"# Re(CKM) at the SUSY scale"
   Do i1=1,3
    Do i2=1,3
     Write(io_L,107) i1,i2,Real(CKM_Q(i1,i2),dp),"# Re(V_"//Bu(i1)//Bu(i2)//")"
    End Do
   End Do
   If (Maxval(Abs(Aimag(CKM_Q))).Gt.0._dp) Then
    Write(io_L,106) "Block IMVCKM Q=",Q,"# Im(CKM) at the SUSY scale"
    Do i1=1,3
     Do i2=1,3
      Write(io_L,107) i1,i2,Aimag(CKM_Q(i1,i2)),"# Im(V_"//Bu(i1)//Bu(i2)//")"
     End Do
    End Do
   End If

  End If ! generationmixing

  Write(io_L,106) "Block Ye Q=",Q,"# (SUSY scale)"

  ierr = 0
  If (GenerationMixing) Then
   !-------------------------------------------
   ! check if any off-diagonal term is non-zero
   !-------------------------------------------
   Do i1=1,3
    Do i2=1,3
     If ((i1.Ne.i2).And.(Abs(Y_l(i2,i1)).Ne.0._dp)) ierr = ierr + 1
    End Do
   End Do
  End If

  If (ierr.Ne.0) Then
   Do i1=1,3
    Do i2=1,3
     Write(io_L,105) i2,i1,Real(Y_l(i2,i1),dp),"# Y_(l,"//bu(i2)//bu(i1)//")"  
    End Do
   End Do
  Else 
   Write(io_L,107) 1,1,Real(y_l(1,1),dp), "# Y_e(Q)^DRbar"
   Write(io_L,107) 2,2,Real(y_l(2,2),dp), "# Y_mu(Q)^DRbar"
   Write(io_L,107) 3,3,Real(y_l(3,3),dp), "# Y_tau(Q)^DRbar"
  End If
  If (Maxval(Abs(Aimag(Y_l))).Gt.0._dp) Then
   Write(io_L,106) "Block IMYe Q=",Q,"# (SUSY scale)"
   If (GenerationMixing) Then
    Do i1=1,3
      Do i2=1,3
      Write(io_L,105) i2,i1,Aimag(Y_l(i2,i1)),"# Im(Y_(l,"//bu(i2)//bu(i1)//"))"
     End Do
    End Do
   Else 
    Write(io_L,107) 1,1,Aimag(y_l(1,1)), "# Im(Y_e)(Q)^DRbar"
    Write(io_L,107) 2,2,Aimag(y_l(2,2)), "# Im(Y_mu)(Q)^DRbar"
    Write(io_L,107) 3,3,Aimag(y_l(3,3)), "# Im(Y_tau)(Q)^DRbar"
   End If
  End If

!   If (GenerationMixing) Then
!    Write(io_L,106) "Block Tu Q=",Q,"# (SUSY scale)"
!    Do i1=1,3
!     Do i2=1,3
!      Write(io_L,105) i2,i1,Real(Au_sckm(i1,i2),dp) &
!                    &  ,"# T_(u,"//bu(i2)//bu(i1)//")"  
!     End Do
!    End Do
!   Else 
   Write(io_L,106) "Block Au Q=",Q,"# (SUSY scale)"
   Write(io_L,107) 1,1,Real(Au_sckm(1,1)/y_u(1,1),dp), "# A_u(Q)^DRbar"
   Write(io_L,107) 2,2,Real(Au_sckm(2,2)/y_u(2,2),dp), "# A_c(Q)^DRbar"
   Write(io_L,107) 3,3,Real(Au_sckm(3,3)/y_u(3,3),dp), "# A_t(Q)^DRbar"
!   End If
!   If (Maxval(Abs(Aimag(Au_sckm))).Gt.0._dp) Then
!    If (GenerationMixing) Then
!     Write(io_L,106) "Block IMTu Q=",Q,"# (SUSY scale)"
!     Do i1=1,3
!       Do i2=1,3
!       Write(io_L,105) i2,i1,Aimag(Au_sckm(i1,i2))  &
!                     & ,"# Im(T_(u,"//bu(i2)//bu(i1)//"))"
!      End Do
!     End Do
!    Else 
!     Write(io_L,106) "Block IMAu Q=",Q,"# (SUSY scale)"
!     Write(io_L,107) 1,1,Aimag(Au_sckm(1,1)/y_u(1,1)), "# Im(A_u)(Q)^DRbar"
!     Write(io_L,107) 2,2,Aimag(Au_sckm(2,2)/y_u(2,2)), "# Im(A_c)(Q)^DRbar"
!     Write(io_L,107) 3,3,Aimag(Au_sckm(3,3)/y_u(3,3)), "# Im(A_t)(Q)^DRbar"
!    End If
!   End If
! 
!   If (GenerationMixing) Then
!    Write(io_L,106) "Block Td Q=",Q,"# (SUSY scale)"
!    Do i1=1,3
!     Do i2=1,3
!      Write(io_L,105) i2,i1,Real(Ad_sckm(i1,i2),dp) &
!                        & ,"# T_(d,"//bu(i2)//bu(i1)//")"  
!     End Do
!    End Do
!   Else 
   Write(io_L,106) "Block Ad Q=",Q,"# (SUSY scale)"
   Write(io_L,107) 1,1,Real(Ad_sckm(1,1)/y_d(1,1),dp), "# A_d(Q)^DRbar"
   Write(io_L,107) 2,2,Real(Ad_sckm(2,2)/y_d(2,2),dp), "# A_s(Q)^DRbar"
   Write(io_L,107) 3,3,Real(Ad_sckm(3,3)/y_d(3,3),dp), "# A_b(Q)^DRbar"
!   End If
!   If (Maxval(Abs(Aimag(Ad_sckm))).Gt.0._dp) Then
!    If (GenerationMixing) Then
!     Write(io_L,106) "Block IMTd Q=",Q,"# (SUSY scale)"
!     Do i1=1,3
!       Do i2=1,3
!       Write(io_L,105) i2,i1,Aimag(Ad_sckm(i1,i2)) &
!                    & ,"# Im(T_(d,"//bu(i2)//bu(i1)//"))"
!      End Do
!     End Do
!    Else 
!     Write(io_L,106) "Block IMAd Q=",Q,"# (SUSY scale)"
!     Write(io_L,107) 1,1,Aimag(Ad_sckm(1,1)/y_d(1,1)), "# Im(A_d)(Q)^DRbar"
!     Write(io_L,107) 2,2,Aimag(Ad_sckm(2,2)/y_d(2,2)), "# Im(A_s)(Q)^DRbar"
!     Write(io_L,107) 3,3,Aimag(Ad_sckm(3,3)/y_d(3,3)), "# Im(A_b)(Q)^DRbar"
!    End If
!   End If
! 
!   ierr = 0
!   If (GenerationMixing) Then
!    !-------------------------------------------
!    ! check if any off-diagonal term is non-zero
!    !-------------------------------------------
!    Do i1=1,3
!     Do i2=1,3
!      If ((i1.Ne.i2).And.(Abs(A_l(i2,i1)).Ne.0._dp)) ierr = ierr + 1
!     End Do
!    End Do
!   End If
! 
!   If (ierr.Ne.0) Then
!    Write(io_L,106) "Block Te Q=",Q,"# (SUSY scale)"
!    Do i1=1,3
!     Do i2=1,3
!      Write(io_L,105) i2,i1,Real(A_l(i2,i1),dp),"# T_(l,"//bu(i2)//bu(i1)//")"  
!     End Do
!    End Do
!   Else 
   Write(io_L,106) "Block Ae Q=",Q,"# (SUSY scale)"
   Write(io_L,107) 1,1,Real(A_l(1,1)/y_l(1,1),dp), "# A_e(Q)^DRbar"
   Write(io_L,107) 2,2,Real(A_l(2,2)/y_l(2,2),dp), "# A_mu(Q)^DRbar"
   Write(io_L,107) 3,3,Real(A_l(3,3)/y_l(3,3),dp), "# A_tau(Q)^DRbar"
!   End If
!   If (Maxval(Abs(Aimag(A_l))).Gt.0._dp) Then
!    If (ierr.Ne.0) Then
!     Write(io_L,106) "Block IMTe Q=",Q,"# (SUSY scale)"
!     Do i1=1,3
!       Do i2=1,3
!       Write(io_L,105) i2,i1,Aimag(A_l(i2,i1)),"# Im(T_(l,"//bu(i2)//bu(i1)//"))"
!      End Do
!     End Do
!    Else 
!     Write(io_L,106) "Block IMAe Q=",Q,"# (SUSY scale)"
!     Write(io_L,107) 1,1,Aimag(A_l(1,1)/y_l(1,1)), "# Im(A_e)(Q)^DRbar"
!     Write(io_L,107) 2,2,Aimag(A_l(2,2)/y_l(2,2)), "# Im(A_mu)(Q)^DRbar"
!     Write(io_L,107) 3,3,Aimag(A_l(3,3)/y_l(3,3)), "# Im(A_tau)(Q)^DRbar"
!    End If
!   End If


  Write(io_L,106) "Block MSOFT Q=",Q,"# soft SUSY breaking masses at Q"
  Write(io_L,104) 1,Real(Mi(1),dp),"# M_1"
  Write(io_L,104) 2,Real(Mi(2),dp),"# M_2"
  Write(io_L,104) 3,Real(Mi(3),dp),"# M_3"
  Write(io_L,104) 21,M2_H(1),"# M^2_(H,d)"
  Write(io_L,104) 22,M2_H(2),"# M^2_(H,u)"

  Write(io_L,104) 31,Sqrt(Real(M2_L(1,1),dp)),"# M_(L,11)"
  Write(io_L,104) 32,Sqrt(Real(M2_L(2,2),dp)),"# M_(L,22)"
  Write(io_L,104) 33,Sqrt(Real(M2_L(3,3),dp)),"# M_(L,33)"
  Write(io_L,104) 34,Sqrt(Real(M2_E(1,1),dp)),"# M_(E,11)"
  Write(io_L,104) 35,Sqrt(Real(M2_E(2,2),dp)),"# M_(E,22)"
  Write(io_L,104) 36,Sqrt(Real(M2_E(3,3),dp)),"# M_(E,33)"
  Write(io_L,104) 41,Sqrt(Real(M2Q_SCKM(1,1),dp)),"# M_(Q,11)"
  Write(io_L,104) 42,Sqrt(Real(M2Q_SCKM(2,2),dp)),"# M_(Q,22)"
  Write(io_L,104) 43,Sqrt(Real(M2Q_SCKM(3,3),dp)),"# M_(Q,33)"
  Write(io_L,104) 44,Sqrt(Real(M2U_SCKM(1,1),dp)),"# M_(U,11)"
  Write(io_L,104) 45,Sqrt(Real(M2U_SCKM(2,2),dp)),"# M_(U,22)"
  Write(io_L,104) 46,Sqrt(Real(M2U_SCKM(3,3),dp)),"# M_(U,33)"
  Write(io_L,104) 47,Sqrt(Real(M2D_SCKM(1,1),dp)),"# M_(D,11)"
  Write(io_L,104) 48,Sqrt(Real(M2D_SCKM(2,2),dp)),"# M_(D,22)"
  Write(io_L,104) 49,Sqrt(Real(M2D_SCKM(3,3),dp)),"# M_(D,33)"

  If (GenerationMixing) Then
   Write(io_L,106) "Block MSL2 Q=",Q,"# M^2_L soft SUSY breaking masses at Q"
   Do i2=1,3
    Do i1=1,3
     Write(io_L,105) i2,i1,Real(M2_L(i2,i1),dp)  &
                  & ,"# M^2_(L,"//bu(i2)//bu(i1)//")"  
    End Do
   End Do
   If (Maxval(Abs(Aimag(M2_L))).Gt.0._dp) Then
    Write(io_L,106) "Block IMMSL2 Q=",Q  &
                  & ,"# Im(M^2_L) soft SUSY breaking masses at Q"
    Do i2=1,3
     Do i1=1,3
      Write(io_L,105) i2,i1,Aimag(M2_L(i2,i1)) &
                 &   ,"# Im(M^2_(L,"//bu(i2)//bu(i1)//"))"  
     End Do
    End Do
   End If

   Write(io_L,106) "Block MSE2 Q=",Q,"# M^2_E soft SUSY breaking masses at Q"
   Do i2=1,3
    Do i1=1,3
     Write(io_L,105) i2,i1,Real(M2_E(i2,i1),dp)  &
                  & ,"# M^2_(E,"//bu(i2)//bu(i1)//")"  
    End Do
   End Do
   If (Maxval(Abs(Aimag(M2_E))).Gt.0._dp) Then
    Write(io_L,106) "Block IMMSE2 Q=",Q  &
                  & ,"# Im(M^2_E) soft SUSY breaking masses at Q"
    Do i2=1,3
     Do i1=1,3
      Write(io_L,105) i2,i1,Aimag(M2_E(i2,i1)) &
                 &   ,"# Im(M^2_(E,"//bu(i2)//bu(i1)//"))"  
     End Do
    End Do
   End If

   Write(io_L,106) "Block MSQ2 Q=",Q,"# M^2_Q soft SUSY breaking masses at Q"
   Do i2=1,3
    Do i1=1,3
     Write(io_L,105) i2,i1,Real(M2Q_SCKM(i2,i1),dp) &
                   & ,"# M^2_(Q,"//bu(i2)//bu(i1)//")"  
    End Do
   End Do
   If (Maxval(Abs(Aimag(M2Q_SCKM))).Gt.0._dp) Then
    Write(io_L,106) "Block IMMSQ2 Q=",Q  &
                  & ,"# Im(M^2_Q) soft SUSY breaking masses at Q"
    Do i2=1,3
     Do i1=1,3
      Write(io_L,105) i2,i1,Aimag(M2Q_SCKM(i2,i1)) &
                 &   ,"# Im(M^2_(Q,"//bu(i2)//bu(i1)//"))"  
     End Do
    End Do
   End If

   Write(io_L,106) "Block MSU2 Q=",Q,"# M^2_U soft SUSY breaking masses at Q"
   Do i2=1,3
    Do i1=1,3
     Write(io_L,105) i2,i1,Real(M2U_SCKM(i1,i2),dp)  &
                  & ,"# M^2_(U,"//bu(i2)//bu(i1)//")"  
    End Do
   End Do
   If (Maxval(Abs(Aimag(M2U_SCKM))).Gt.0._dp) Then
    Write(io_L,106) "Block IMMSU2 Q=",Q  &
                  & ,"# Im(M^2_U) soft SUSY breaking masses at Q"
    Do i2=1,3
     Do i1=1,3
      Write(io_L,105) i2,i1,Aimag(M2U_SCKM(i1,i2)) &
                 &   ,"# Im(M^2_(U,"//bu(i2)//bu(i1)//"))"  
     End Do
    End Do
   End If

   Write(io_L,106) "Block MSD2 Q=",Q,"# M^2_D soft SUSY breaking masses at Q"
   Do i2=1,3
    Do i1=1,3
     Write(io_L,105) i2,i1,Real(M2D_SCKM(i1,i2),dp)  &
                  & ,"# M^2_(D,"//bu(i2)//bu(i1)//")"  
    End Do
   End Do
   If (Maxval(Abs(Aimag(M2D_SCKM))).Gt.0._dp) Then
    Write(io_L,106) "Block IMMSD2 Q=",Q  &
                  & ,"# Im(M^2_D) soft SUSY breaking masses at Q"
    Do i2=1,3
     Do i1=1,3
      Write(io_L,105) i2,i1,Aimag(M2D_SCKM(i1,i2)) &
                 &   ,"# Im(M^2_(D,"//bu(i2)//bu(i1)//"))"  
     End Do
    End Do
   End If

  End If

  If (HighScaleModel.Eq."NMSSM") Then
   Write(io_L,104) 61,Real(h0,dp),"# lambda"
   Write(io_L,104) 62,0.5_dp*Real(lam,dp),"# kappa"
   Write(io_L,104) 63,Real(ao_h0,dp),"# A_lambda"
   Write(io_L,104) 64,Real(Ao_lam,dp),"# A_kappa"
   Write(io_L,104) 65,Real(oosqrt2*vP*h0,dp),"# mu_eff"

  Else If (HighScaleModel.Eq."RPexplicit") Then
   Write(io_L,106) "Block RVKAPPA Q=",Q,"# bilinear RP parameters at Q"
   Write(io_L,102) 1,Real(eps(1),dp),"# epsilon_1"
   Write(io_L,102) 2,Real(eps(2),dp),"# epsilon_2"
   Write(io_L,102) 3,Real(eps(3),dp),"# epsilon_3"
   If (Maxval(Abs(Aimag(eps))).Gt.0._dp) Then
    Write(io_L,106) "Block IMRVKAPPA Q=",Q,"# imaginary parts of bilinear RP parameters at Q"
    Write(io_L,102) 1,Aimag(eps(1)),"# Im(epsilon_1)"
    Write(io_L,102) 2,Aimag(eps(2)),"# Im(epsilon_2)"
    Write(io_L,102) 3,Aimag(eps(3)),"# Im(epsilon_3)"
   End If

   Write(io_L,106) "Block RVD Q=",Q,"# bilinear RP parameters at Q"
   Write(io_L,102) 1,Real(Beps(1),dp),"# Re( B_1 epsilon_1)"
   Write(io_L,102) 2,Real(Beps(2),dp),"# Re( B_2 epsilon_2)"
   Write(io_L,102) 3,Real(Beps(3),dp),"# Re( B_3 epsilon_3)"
   If (Maxval(Abs(Aimag(eps))).Gt.0._dp) Then
    Write(io_L,106) "Block IMRVD Q=",Q,"# imaginary parts of bilinear RP parameters at Q"
    Write(io_L,102) 1,Aimag(Beps(1)),"# Im( B_1 epsilon_1)"
    Write(io_L,102) 2,Aimag(Beps(2)),"# Im( B_2 epsilon_2)"
    Write(io_L,102) 3,Aimag(Beps(3)),"# Im( B_3 epsilon_3)"
   End If

   Write(io_L,106) "Block RVSNVEV Q=",Q,"# sneutrino vevs at Q"
   Write(io_L,102) 1,vevL(1),"# v_L_1"
   Write(io_L,102) 2,vevL(2),"# v_L_2"
   Write(io_L,102) 3,vevL(3),"# v_L_3"

   If (Maxval(Abs(Rp_lam)).Gt.0._dp) Then
    Write(io_L,106) "Block RVLAMPBDA Q=",Q,"# lambda_ijk at Q"
    Do i1=1,3
     Do i2=1,3
      Do i3=1,3
       Write(io_L,109) i1,i2,i3,Real(Rp_lam(i1,i2,i3),dp) &
                     & ,"# lambda_"//Bu(i1)//Bu(i2)//Bu(i3)
      End Do
     End Do
    End Do
   End If
   If (Maxval(Abs(Aimag(Rp_lam))).Gt.0._dp) Then
    Write(io_L,106) "Block IMRVLAMPBDA Q=",Q,"# Im(lambda_ijk) at Q"
    Do i1=1,3
     Do i2=1,3
      Do i3=1,3
       Write(io_L,109) i1,i2,i3,Aimag(Rp_lam(i1,i2,i3)) &
            & ,"# Im(lambda_"//Bu(i1)//Bu(i2)//Bu(i3)//")"
      End Do
     End Do
    End Do
   End If

   If (Maxval(Abs(Rp_lamp)).Gt.0._dp) Then
    Write(io_L,106) "Block RVLAMPBDAP Q=",Q,"# lambda'_ijk at Q"
    Do i1=1,3
     Do i2=1,3
      Do i3=1,3
       Write(io_L,109) i1,i2,i3,Real(Rp_lamp(i1,i2,i3),dp) &
                     & ,"# lambda'_"//Bu(i1)//Bu(i2)//Bu(i3)
      End Do
     End Do
    End Do
   End If
   If (Maxval(Abs(Aimag(Rp_lam))).Gt.0._dp) Then
    Write(io_L,106) "Block IMRVLAMPBDAP Q=",Q,"# Im(lambda'_ijk) at Q"
    Do i1=1,3
     Do i2=1,3
      Do i3=1,3
       Write(io_L,109) i1,i2,i3,Aimag(Rp_lamp(i1,i2,i3)) &
            & ,"# Im(lambda'_"//Bu(i1)//Bu(i2)//Bu(i3)//")"
      End Do
     End Do
    End Do
   End If

   Write(io_L,100) "Block SPhenoRP  # additional RP parameters"
   Write(io_L,102) 4,Lam_Ex(1),"# Lambda_1 = v_d epsilon_1 + mu v_L1"
   Write(io_L,102) 5,Lam_Ex(2),"# Lambda_2 = v_d epsilon_2 + mu v_L2"
   Write(io_L,102) 6,Lam_Ex(3),"# Lambda_3 = v_d epsilon_3 + mu v_L3"
   Write(io_L,102) 7,(mN7(3)**2-mN7(2)**2)*1.e18_dp,"# m^2_atm [eV^2]"
   Write(io_L,102) 8,(mN7(2)**2-mN7(1)**2)*1.e18_dp,"# m^2_sol [eV^2]"
   Write(io_L,102) 9,Abs(N7(3,6)/N7(3,7))**2,"# tan^2 theta_atm"
   Write(io_L,102) 10,Abs(N7(2,5)/N7(1,5))**2,"# tan^2 theta_sol"
   Write(io_L,102) 11,Abs(N7(3,5))**2,"# U_e3^2"
   Write(io_L,102) 15,vevSM(1),"# v_d"
   Write(io_L,102) 16,vevSM(2),"# v_u"
  End If


   Write(io_L,100) "Block MASS  # Mass spectrum"
   Write(io_L,100) "#   PDG code      mass          particle"
   Write(io_L,102) id_u(2),mf_u(2),"# m_c(m_c), MSbar"
   Write(io_L,102) id_d(3),mf_d(3),"# m_b(m_b), MSbar"
   Write(io_L,102) id_u(3),mf_u(3),"# m_t(pole)"
   Write(io_L,102) id_Z,mZ,"# m_Z(pole)"
   Write(io_L,102) id_W,mW,"# W+"
   If (HighScaleModel.Ne."RPexplicit") &
        & Write(io_L,102) id_l(3),mf_l(3),"# m_tau(pole)"
   If (HighScaleModel.Eq."NMSSM") Then
    Write(io_L,102) 25,mS03(1),"# leightest neutral scalar" 
    Write(io_L,102) 35,mS03(2),"# second neutral scalar" 
    Write(io_L,102) 45,mS03(3),"# third neutral scalar" 
    Write(io_L,102) 36,mP03(2),"# leighter pseudoscalar" 
    Write(io_L,102) 46,mP03(3),"# heavier pseudoscalar" 
    Write(io_L,102) 37,mSpm(2),"# H+"
   Else If (HighScaleModel.Eq."RPexplicit") Then
!    Write(io_L,102) 12,mN7(1),"# lightest neutrino"
!    Write(io_L,102) 14,mN7(2),"# second lightest neutrino"
!    Write(io_L,102) 16,mN7(3),"# heaviest neutrino"
    Write(io_L,102) id_s0(1),mS05(1),"# leightest neutral scalar" 
    Write(io_L,102) id_s0(2),mS05(2),"# 2nd neutral scalar" 
    Write(io_L,102) id_s0(3),mS05(3),"# 3rd neutral scalar" 
    Write(io_L,102) id_s0(4),mS05(4),"# 4th neutral scalar" 
    Write(io_L,102) id_s0(5),mS05(5),"# 5th neutral scalar" 
    Write(io_L,102) id_P0(1),mP05(2),"# leightest pseudoscalar" 
    Write(io_L,102) id_P0(2),mP05(3),"# 2nd pseudoscalar" 
    Write(io_L,102) id_P0(3),mP05(4),"# 3rd pseudoscalar" 
    Write(io_L,102) id_p0(4),mP05(5),"# 4th pseudoscalar"
    Write(io_L,102) Abs(id_sp(1)),mSpm8(2),"# leightest charged scalar" 
    Write(io_L,102) Abs(id_sp(2)),mSpm8(3),"# 2nd charged scalar" 
    Write(io_L,102) Abs(id_sp(3)),mSpm8(4),"# 3rd charged scalar" 
    Write(io_L,102) Abs(id_sp(4)),mSpm8(5),"# 4th charged scalar" 
    Write(io_L,102) Abs(id_sp(5)),mSpm8(6),"# 5th charged scalar" 
    Write(io_L,102) Abs(id_sp(6)),mSpm8(7),"# 6th charged scalar" 
    Write(io_L,102) Abs(id_sp(7)),mSpm8(8),"# 7th charged scalar" 
    
   Else
    Write(io_L,102) 25,mS0(1),"# h0" 
    Write(io_L,102) 35,mS0(2),"# H0" 
    Write(io_L,102) 36,mP0(2),"# A0" 
    Write(io_L,102) 37,mSpm(2),"# H+"
   End If
! squarks

  If (GenerationMixing) Then
   If (Use_Flavour_States) Then ! using flavour ordering, old fashionnd
     i_zaehl = 1
     Do i1=1,6
      MaxCont = Maxval(Abs( RDsq_ckm(i1,:))**2)
      If (Abs(Abs( RDsq_ckm(i1,1))**2-maxCont).Lt. 10._dp * Epsilon(1._dp)) Then
       id_sd(i1) = 1000001
       c_sd(i1) = "~d_L"
      Else If (Abs(Abs( RDsq_ckm(i1,2))**2-maxCont).Lt.10._dp*Epsilon(1._dp)) Then
       id_sd(i1) = 1000003
       c_sd(i1) = "~s_L-"
      Else If (Abs(Abs( RDsq_ckm(i1,4))**2-maxCont).Lt.10._dp*Epsilon(1._dp)) Then
       id_sd(i1) = 2000001
       c_sd(i1) = "~d_R"
      Else If (Abs(Abs( RDsq_ckm(i1,5))**2-maxCont).Lt.10._dp*Epsilon(1._dp)) Then
       id_sd(i1) = 2000003
       c_sd(i1) = "~s_R"
      Else
       id_sd(i1) = 1000000 * i_zaehl + 5
       c_sd(i1) = "~b_"//bu(i_zaehl)//"-"
       i_zaehl = I_zaehl + 1
      End If
      Write(io_L,102) id_sd(i1),msdown(i1),"# "//Trim(c_sd(i1))
     End Do

     i_zaehl = 1
     Do i1=1,6
      MaxCont = Maxval(Abs( RUsq_ckm(i1,:))**2)
      If (Abs(Abs( RUsq_ckm(i1,1))**2-maxCont).Lt. 10._dp * Epsilon(1._dp)) Then
       id_su(i1) = 1000002
       c_su(i1) = "~u_L"
      Else If (Abs(Abs( RUsq_ckm(i1,2))**2-maxCont).Lt.10._dp*Epsilon(1._dp)) Then
       id_su(i1) = 1000004
       c_su(i1) = "~c_L-"
      Else If (Abs(Abs( RUsq_ckm(i1,4))**2-maxCont).Lt.10._dp*Epsilon(1._dp)) Then
       id_su(i1) = 2000002
       c_su(i1) = "~u_R"
      Else If (Abs(Abs( RUsq_ckm(i1,5))**2-maxCont).Lt.10._dp*Epsilon(1._dp)) Then
       id_su(i1) = 2000004
       c_su(i1) = "~c_R"
      Else
       id_su(i1) = 1000000 * i_zaehl + 6
       c_su(i1) = "~t_"//bu(i_zaehl)//"-"
       i_zaehl = I_zaehl + 1
      End If
      Write(io_L,102) id_su(i1),msup(i1),"# "//Trim(c_su(i1))
     End Do
   
    Else ! use mass ordering

     id_sd(1) = 1000001
     id_sd(2) = 1000003
     id_sd(3) = 1000005
     id_sd(4) = 2000001
     id_sd(5) = 2000003
     id_sd(6) = 2000005
     Do i1=1,6
      c_sd(i1) = "~d_"//Bu(i1)
      Write(io_L,102) id_sd(i1),msdown(i1),"# "//Trim(c_sd(i1))
     End Do

     id_su(1) = 1000002
     id_su(2) = 1000004
     id_su(3) = 1000006
     id_su(4) = 2000002
     id_su(5) = 2000004
     id_su(6) = 2000006
     Do i1=1,6
      c_su(i1) = "~u_"//Bu(i1)
      Write(io_L,102) id_su(i1),msup(i1),"# "//Trim(c_su(i1))
     End Do
    End If
  Else ! .not.GenerationMixing

   If (Abs(rsdown(1,1)).Gt.0.5_dp) Then
    Write(io_L,102) 1000001,msdown(1),"# ~d_L"
    Write(io_L,102) 2000001,msdown(2),"# ~d_R"
    id_sd(1) = 1000001
    id_sd(2) = 2000001
    c_sd(1) = "~d_L"
    c_sd(2) = "~d_R"
   Else
    Write(io_L,102) 1000001,msdown(2),"# ~d_L"
    Write(io_L,102) 2000001,msdown(1),"# ~d_R"
    id_sd(2) = 1000001
    id_sd(1) = 2000001
    c_sd(2) = "~d_L"
    c_sd(1) = "~d_R"
   End If
   If (Abs(rsup(1,1)).Gt.0.5_dp) Then
    Write(io_L,102) 1000002,msup(1),"# ~u_L"
    Write(io_L,102) 2000002,msup(2),"# ~u_R"
    id_su(1) = 1000002
    id_su(2) = 2000002
    c_su(1) = "~u_L"
    c_su(2) = "~u_R"
   Else
    Write(io_L,102) 1000002,msup(2),"# ~u_L"
    Write(io_L,102) 2000002,msup(1),"# ~u_R"
    id_su(2) = 1000002
    id_su(1) = 2000002
    c_su(2) = "~u_L"
    c_su(1) = "~u_R"
   End If
   If (Abs(rsdown(3,3)).Gt.0.5_dp) Then
    Write(io_L,102) 1000003,msdown(3),"# ~s_L"
    Write(io_L,102) 2000003,msdown(4),"# ~s_R"
    id_sd(3) = 1000003
    id_sd(4) = 2000003
    c_sd(3) = "~s_L"
    c_sd(4) = "~s_R"
   Else
    Write(io_L,102) 1000003,msdown(4),"# ~s_L"
    Write(io_L,102) 2000003,msdown(3),"# ~s_R"
    id_sd(4) = 1000003
    id_sd(3) = 2000003
    c_sd(4) = "~s_L"
    c_sd(3) = "~s_R"
   End If
   If (Abs(rsup(3,3)).Gt.0.5_dp) Then
    Write(io_L,102) 1000004,msup(3),"# ~c_L"
    Write(io_L,102) 2000004,msup(4),"# ~c_R"
    id_su(3) = 1000004
    id_su(4) = 2000004
    c_su(3) = "~c_L"
    c_su(4) = "~c_R"
   Else
    Write(io_L,102) 1000004,msup(4),"# ~c_L"
    Write(io_L,102) 2000004,msup(3),"# ~c_R"
    id_su(4) = 1000004
    id_su(3) = 2000004
    c_su(4) = "~c_L"
    c_su(3) = "~c_R"
   End If
   Write(io_L,102) 1000005,msdown(5),"# ~b_1"
   Write(io_L,102) 2000005,msdown(6),"# ~b_2"
   id_sd(5) = 1000005
   id_sd(6) = 2000005
   c_sd(5) = "~b_1"
   c_sd(6) = "~b_2"
   Write(io_L,102) 1000006,msup(5),"# ~t_1"
   Write(io_L,102) 2000006,msup(6),"# ~t_2"
   id_su(5) = 1000006
   id_su(6) = 2000006
   c_su(5) = "~t_1"
   c_su(6) = "~t_2"
  End If ! GenerationMixing
   
   If (HighScaleModel.Ne."RPexplicit") Then

! sleptons

    If (GenerationMixing) Then
     If (Use_Flavour_States) Then ! using flavour ordering, old fashionnd
 
      Do i1=1,3
       MaxCont = Maxval(Abs(Rsneut(i1,:))**2)
       If (Abs(Rsneut(i1,1))**2.Eq.maxCont) Then
        id_snu(i1) = 1000012
        c_snu(i1) = "~nu_eL"
       Else If (Abs(Rsneut(i1,2))**2.Eq.maxCont) Then
        id_snu(i1) = 1000014
        c_snu(i1) = "~nu_muL"
       Else
        id_snu(i1) = 1000016
        c_snu(i1) = "~nu_tauL"
       End If
       Write(io_L,102) id_snu(i1),msneut(i1),"# "//Trim(c_snu(i1))
      End Do

      i_zaehl = 1
      mat6R = Abs(RSlepton)
      Do i1=1,6
       MaxCont = Maxval(mat6R)
       Call FindPosition(6, mat6R, MaxCont, ii, jj)
       Select Case (jj)
       Case(1)
        id_sle(ii) = 1000011
        c_sle(ii) = "~e_L-"
        c_slep(ii) = "~e_L+"
       Case(2)
        id_sle(ii) = 1000013
        c_sle(ii) = "~mu_L-"
        c_slep(ii) = "~mu_L+"
       Case(4)
        id_sle(ii) = 2000011
        c_sle(ii) = "~e_R-"
        c_slep(ii) = "~e_R+"
       Case(5)
        id_sle(ii) = 2000013
        c_sle(ii) = "~mu_R-"
        c_slep(ii) = "~mu_R+"
       Case default
        id_sle(ii) = 1000000 * i_zaehl + 15
        c_sle(ii) = "~tau_"//bu(i_zaehl)//"-"
        c_slep(ii) = "~tau_"//bu(i_zaehl)//"+"
        i_zaehl = I_zaehl + 1
       End Select
       mat6R(ii,:) = 0._dp
       mat6R(:,jj) = 0._dp
      End Do
      Do ii=1,6
       Write(io_L,102) id_sle(ii),mslepton(ii),"# "//Trim(c_sle(ii))
      End Do

     Else ! using mass ordering

      id_snu(1) = 1000012
      id_snu(2) = 1000014
      id_snu(3) = 1000016
      Do i1=1,3
       c_snu(i1) = "~nu_"//Bu(i1)
       Write(io_L,102) id_snu(i1),mSneut(i1),"# "//Trim(c_snu(i1))
      End Do

      id_sle(1) = 1000011
      id_sle(2) = 1000013
      id_sle(3) = 1000015
      id_sle(4) = 2000011
      id_sle(5) = 2000013
      id_sle(6) = 2000015
      Do i1=1,6
       c_sle(i1) = "~l_"//Bu(i1)
       c_slep(i1) = "~l+_"//Bu(i1)
       Write(io_L,102) id_sle(i1),mSlepton(i1),"# "//Trim(c_sle(i1))
      End Do
     End If

    Else ! .not.GenerationMixing

     id_snu = (/ 1000012, 1000014, 1000016 /)
     If (Abs(rslepton(1,1)).Gt.0.5_dp) Then
      Write(io_L,102) 1000011,mslepton(1),"# ~e_L-"
      Write(io_L,102) 2000011,mslepton(2),"# ~e_R-"
      id_sle(1) = 1000011
      id_sle(2) = 2000011
      c_sle(1) = "~e_L-"
      c_sle(2) = "~e_R-"
      c_slep(1) = "~e_L+"
      c_slep(2) = "~e_R+"
     Else
      Write(io_L,102) 1000011,mslepton(2),"# ~e_L-"
      Write(io_L,102) 2000011,mslepton(1),"# ~e_R-"
      id_sle(2) = 1000011
      id_sle(1) = 2000011
      c_sle(2) = "~e_L-"
      c_sle(1) = "~e_R-"
      c_slep(2) = "~e_L+"
      c_slep(1) = "~e_R+"
     End If
     Write(io_L,102) 1000012,msneut(1),"# ~nu_eL"
     c_snu(1) = "~nu_eL"
     If (Abs(rslepton(3,3)).Gt.0.5_dp) Then
      Write(io_L,102) 1000013,mslepton(3),"# ~mu_L-"
      Write(io_L,102) 2000013,mslepton(4),"# ~mu_R-"
      id_sle(3) = 1000013
      id_sle(4) = 2000013
      c_sle(3) = "~mu_L-"
      c_sle(4) = "~mu_R-"
      c_slep(3) = "~mu_L+"
      c_slep(4) = "~mu_R+"
     Else
      Write(io_L,102) 1000013,mslepton(4),"# ~mu_L-"
      Write(io_L,102) 2000013,mslepton(3),"# ~mu_R-"
      id_sle(4) = 1000013
      id_sle(3) = 2000013
      c_sle(4) = "~mu_L-"
      c_sle(3) = "~mu_R-"
      c_slep(4) = "~mu_L+"
      c_slep(3) = "~mu_R+"
     End If
     Write(io_L,102) 1000014,msneut(2),"# ~nu_muL"
     c_snu(2) = "~nu_muL"
     Write(io_L,102) 1000015,mslepton(5),"# ~tau_1-"
     Write(io_L,102) 2000015,mslepton(6),"# ~tau_2-"
     id_sle(5) = 1000015
     id_sle(6) = 2000015
     c_sle(5) = "~tau_1-"
     c_sle(6) = "~tau_2-"
     c_slep(5) = "~tau_1+"
     c_slep(6) = "~tau_2+"
     Write(io_L,102) 1000016,msneut(3),"# ~nu_tauL"
     c_snu(3) = "~nu_tauL"
    End If
   End If ! GenerationMixing

 ! gauginos/higgsinos
   Write(io_L,102) 1000021,mglu,"# ~g"
   ! checking for negative sign

    Do i1=1,4
     If (Sum(Abs(Real(N(i1,:)))).Lt.0.1_dp) Then
      mNr(i1) = - mN(i1)
      nr(i1,1:4) = Aimag(n(i1,:))
     Else   
      mNr(i1) =  mN(i1)
      nr(i1,1:4) = Real(n(i1,:),dp)
     End If
    End Do

    Write(io_L,102) 1000022,mnr(1),"# ~chi_10" 
    Write(io_L,102) 1000023,mnr(2),"# ~chi_20" 
    Write(io_L,102) 1000025,mnr(3),"# ~chi_30" 
    Write(io_L,102) 1000035,mnr(4),"# ~chi_40" 
    Write(io_L,102) 1000024,mc(1),"# ~chi_1+" 
    Write(io_L,102) 1000037,mc(2),"# ~chi_2+"

   If (Maxval(MnuR).Gt.0._dp) Then
    Write(io_L,100) "# masses of right handed neutrinos"
    Write(io_L,100) "Block MnuR"
    Write(io_L,102) 1,mnur(1),"# m_nu_R_1"
    Write(io_L,102) 2,mnur(2),"# m_nu_R_2"
    Write(io_L,102) 3,mnur(3),"# m_nu_R_3"
   End If

! Mixing matrices 
  Write(io_L,100) "# Higgs mixing"
   Write(io_L,100) "Block alpha # Effective Higgs mixing angle"
   Write(io_L,108) -Asin(RS0(1,1)),"# alpha"
   Write(io_L,103) "Block Hmix Q=",Q, "# Higgs mixing parameters"
   Write(io_L,104) 1,Real(mu,dp),"# mu"
   Write(io_L,104) 2,tanb_Q,"# tan[beta](Q)"
   Write(io_L,104) 3,vev_Q,"# v(Q)"
   Write(io_L,104) 4,mA2_Q,"# m^2_A(Q)"

!--------------------
! add correct order
!--------------------


   T3 = 0.5_dp
   Yl = 1._dp / 3._dp
   Yr = -4._dp / 3._dp
   Ml = M2Q_sckm(3,3)
   Mr = M2U_sckm(3,3)
   A = Au_sckm(3,3)
   Y = Yu(3)
   Call SfermionMass(Ml, Mr, A, mu, vevSM, Y, T3, Yl, Yr, g, gp, kont &
                    &, msf, msf2, Rsf)
   Write(io_L,100) "Block stopmix  # stop mixing matrix"
   Write(io_L,105) 1,1,Real(Rsf(1,1),dp),"# R_st(1,1)"
   Write(io_L,105) 1,2,Real(Rsf(1,2),dp),"# R_st(1,2)"
   Write(io_L,105) 2,1,Real(Rsf(2,1),dp),"# R_st(2,1)"
   Write(io_L,105) 2,2,Real(Rsf(2,2),dp),"# R_st(2,2)"

   T3 = -0.5_dp
   Yl = 1._dp / 3._dp
   Yr = 2._dp / 3._dp
   Ml = M2Q_sckm(3,3)
   Mr = M2d_sckm(3,3)
   A = Ad_sckm(3,3)
   Y = Yd(3)
   Call SfermionMass(Ml, Mr, A, mu, vevSM, Y, T3, Yl, Yr, g, gp, kont &
                    &, msf, msf2, Rsf)
   Write(io_L,100) "Block sbotmix  # sbottom mixing matrix"
   Write(io_L,105) 1,1,Real(Rsf(1,1),dp),"# R_sb(1,1)"
   Write(io_L,105) 1,2,Real(Rsf(1,2),dp),"# R_sb(1,2)"
   Write(io_L,105) 2,1,Real(Rsf(2,1),dp),"# R_sb(2,1)"
   Write(io_L,105) 2,2,Real(Rsf(2,2),dp),"# R_sb(2,2)"

   T3 = -0.5_dp
   Yl = -1._dp 
   Yr = 2._dp
    Ml = M2_L(3,3)
    Mr = M2_E(3,3)
    A = A_l(3,3)
    Y = Y_l(3,3)
    Call SfermionMass(Ml, Mr, A, mu, vevSM, Y, T3, Yl, Yr, g, gp, kont &
                    &, msf, msf2, Rsf)
    Write(io_L,100) "Block staumix  # stau mixing matrix"
   Write(io_L,105) 1,1,Real(Rsf(1,1),dp),"# R_sta(1,1)"
   Write(io_L,105) 1,2,Real(Rsf(1,2),dp),"# R_sta(1,2)"
   Write(io_L,105) 2,1,Real(Rsf(2,1),dp),"# R_sta(2,1)"
   Write(io_L,105) 2,2,Real(Rsf(2,2),dp),"# R_sta(2,2)"

   Write(io_L,100) "Block Nmix  # neutralino/neutrino mixing matrix"
   Do i1=1,n_n
    Do i2=1,n_n
     name = "# N("//Char(48+i1)//","//Char(48+i2)//")"
     Write(io_L,105) i1,i2,nr(i1,i2),name
    End Do
   End Do

   Write(io_L,100) "Block Umix  # chargino U mixing matrix"
   Write(io_L,105) 1,1,Real(U(1,1),dp),"# U(1,1)"
   Write(io_L,105) 1,2,Real(U(1,2),dp),"# U(1,2)"
   Write(io_L,105) 2,1,Real(U(2,1),dp),"# U(2,1)"
   Write(io_L,105) 2,2,Real(U(2,2),dp),"# U(2,2)"
   Write(io_L,100) "Block Vmix  # chargino V mixing matrix"
   Write(io_L,105) 1,1,Real(V(1,1),dp),"# V(1,1)"
   Write(io_L,105) 1,2,Real(V(1,2),dp),"# V(1,2)"
   Write(io_L,105) 2,1,Real(V(2,1),dp),"# V(2,1)"
   Write(io_L,105) 2,2,Real(V(2,2),dp),"# V(2,2)"

 99 Format(1x,i5,3x,a)
100 Format(a)
101 Format(2x,i3,2x,1P,e16.8,2x,a) 
102 Format(1x,i9,3x,1P,e16.8,2x,a)
103 Format(a13,1P,e16.8,2x,a)
104 Format(i4,2x,1P,e16.8,2x,a)
105 Format(1x,2i3,3x,1P,e16.8,3x,a)
106 Format(a,1P,e16.8,2x,a)
107 Format(2i3,3x,1P,e16.8,3x,a)
108 Format(9x,1P,E16.8,0P,3x,a)
109 Format(1x,3i3,3x,1P,e16.8,3x,a)
  close(65)
  

!   if (f_c.lt.10) then
!    f_n = "omg000"//bu(f_c)//".out"
!   else if (f_c.lt.100) then
!    k1 = f_c/10
!    f_n = "omg00"//bu(k1)//bu(f_c-k1*10)//".out"
!   else if (f_c.lt.1000) then
!    k1 = f_c/100
!    k2 = (f_c - k1 * 100)/10
!    f_n = "omg0"//bu(k1)//bu(k2)//bu(f_c-k1*100-k2*10)//".out"
!   else
!    k1 = f_c/1000
!    k2 = (f_c - k1 * 1000)/100
!    k3 = (f_c - k1 * 1000 - k2*100)/10
!    f_n = "omg"//bu(k1)//bu(k2)//bu(k3)//bu(f_c-k1*1000-k2*100-k3*10)//".out"
!   end if   
f_n="test.dat"
!!$   Call system("SPhenoOmg SPheno_1.spc >& "//f_n)
!!$!   Call system("sugomg_f")
!!$   Open(45,file="omg.out",status="old")
!!$   Read(45,*) omega
!!$   close(45)

  Iname = Iname - 1
  
 end subroutine Calculate_Omega_h_sq


End Program SPheno


